#!/usr/bin/perl
#require 5.010;
my $THIS = "compose_qr_pdf.pl";
my $VERSION = "0.01"; # Time-stamp: <2014-01-23T07:34:34Z>

use utf8;
use strict;
use warnings;

use Encode;
use File::Spec::Functions qw(catfile splitpath);
use Storable qw(dclone);
use POSIX qw(ceil);

use Image::Magick;
use PDF::Create;

use Getopt::Long;

#our $INKSCAPE = "c:/Program Files/Inkscape/inkscape.exe";
our $TEMP = "temp.gif";

our $MM_TO_INCH = 1 / 25.4;
our $PT_TO_INCH = 1 / 72.0;
our $PDF_DPI = 72;
our $MODE = "normal";
  # "separate": separate pdf for each ptc programs.
  # "twosides": separate pdf to print by two sides.

our $EXT = "pdf";
our $DPI = 300;
our @SRC_DIR = ();
our $SRC_DIR = "youscout_ptc-20130413";
our $TARGET;
our $DEST_DIR = "qr" . $DPI . $EXT;
our $QR_MARGIN = 8 * $MM_TO_INCH;

our $PAGE = "A4";
our $PAGE_MARGIN_T = 14 * $MM_TO_INCH;
our $PAGE_MARGIN_L = 17 * $MM_TO_INCH;
our $COLS = 2;
our $ROWS = 5;
our $PAPER_W = 49 * $MM_TO_INCH;
our $PAPER_H = 85 * $MM_TO_INCH;
our $CMARGIN = 6 * $MM_TO_INCH;
our $MARGIN = 2 * $MM_TO_INCH;
our $FRAME_W = $PAPER_W + $MARGIN * 2;
our $FRAME_H = $PAPER_H + $MARGIN * 2;
our $AUTHOR = "JRF";
our $REMARK = "(C) 2013 - JRF - Intended to be Public Domain.  ( http://jrf.cocolog-nifty.com/archive/youscout/ptc/index.html )";

Getopt::Long::Configure("bundling", "auto_version");
GetOptions(
	   "dpi:i" => \$DPI,
	   "target:s" => \$TARGET,
	   "youscout-ptc|youscout_ptc:s" => \$SRC_DIR,
	   "separate" => sub { $MODE = "separate"; },
	   "twosides|two-sides" => sub { $MODE = "twosides"; },
	   "h|help" => sub { usage(0); },
	  );

if (@ARGV) {
  @SRC_DIR = @ARGV;
} else {
  foreach my $s ("qrcode_mk", "qrcode_hlp", "qrcode_ysc") {
    push(@SRC_DIR, catfile($SRC_DIR, $s));
  }
}

sub usage {
  print "Usage: $0 [--dpi=DPI] [--separate|--twosides] QRCODES_DIR ..\n";
  exit 0;
}

if (defined $TARGET) {
  $DEST_DIR = $TARGET;
} else {
  $TARGET = "qr${DPI}.${EXT}";
}

my $CARD_WIDTH = int($PAPER_W * $DPI);
my $CARD_HEIGHT = int($PAPER_H * $DPI);

# CPDF == Cards' PDF
our $CPDF_TEMPLATE = {pdf => undef, direction => 0,
		      page => undef, page_num => 0,
		      page_items => 0, last_target => undef};
my $cpdf_front = dclone($CPDF_TEMPLATE);
my $cpdf_back = dclone($CPDF_TEMPLATE);
$cpdf_back->{direction} = 1;

sub cpdf_open {
  my ($cpdf, $filename, %attr) = @_;
  $attr{Author} = $AUTHOR if ! exists $attr{Author};
  $attr{CreationDate} = [ localtime() ] if ! exists $attr{CreationDate};

  $cpdf->{pdf} = new PDF::Create(filename => $filename, %attr);
  $cpdf->{page} = undef;
  $cpdf->{page_num} = 0;
  $cpdf->{last_target} = undef;
}

sub cpdf_close {
  my ($cpdf, $destdir) = @_;
  if (defined $cpdf->{page}) {
    $cpdf->{page_num}++;
    print "$cpdf->{last_target} $cpdf->{page_num}\n";
    $cpdf->{page} = undef;
  }
  $cpdf->{pdf}->close();
}

sub cpdf_shipout {
  my ($cpdf, $src, $target) = @_;
  my $pdf = $cpdf->{pdf};

  if (! defined $cpdf->{page}) {
    $cpdf->{page_items} = 0;
    $cpdf->{page} = $pdf->new_page('MediaBox' => $pdf->get_page_size($PAGE))->new_page();
  }
  $src->Write($TEMP);
  my $img = $pdf->image($TEMP);
  my ($x, $y);
  if ($cpdf->{direction} == 1) {
    $x = $PAGE_MARGIN_L + ($cpdf->{page_items} % $COLS) * ($PAPER_H + $CMARGIN);
    $y = $PAGE_MARGIN_T + int($cpdf->{page_items} / $COLS) * ($PAPER_W + $CMARGIN);
  } else {
    $x = $PAGE_MARGIN_L + ($COLS - ($cpdf->{page_items} % $COLS) - 1) * ($PAPER_H + $CMARGIN);
    $y = $PAGE_MARGIN_T + int($cpdf->{page_items} / $COLS) * ($PAPER_W + $CMARGIN);
  }
  $x = int($x * $PDF_DPI);
  $y = int($y * $PDF_DPI);
  $cpdf->{page}->image(image => $img, xpos => $x, ypos => $y, xscale => $PDF_DPI / $DPI, yscale => $PDF_DPI / $DPI);
  $cpdf->{last_target} = $target;
  $cpdf->{page_items}++;
  if ($cpdf->{page_items} >= $COLS * $ROWS) {
    $cpdf->{page_num}++;
    print "$cpdf->{last_target} $cpdf->{page_num}\n";
    $cpdf->{page} = undef;
  }
}


if (($MODE eq "separate" || $MODE eq "twosides") && ! -d $DEST_DIR) {
  mkdir $DEST_DIR or die "$DEST_DIR: $!";
}

if ($MODE eq "normal") {
  cpdf_open($cpdf_front, $TARGET,
	    Title => "The QR Codes of Youscout PTC");
}

if ($MODE eq "twosides") {
  cpdf_open($cpdf_front, catfile($DEST_DIR, "front.$EXT"),
	    Title => "The QR Codes of Youscout PTC (FRONT)");
  cpdf_open($cpdf_back, catfile($DEST_DIR, "back.$EXT"),
	    Title => "The QR Codes of Youscout PTC (BACK)");
}

foreach my $srcdir (@SRC_DIR) {
  my (undef, undef, $subdir) = splitpath($srcdir);
  my $destdir = catfile($DEST_DIR, $subdir);
#  if (! -d $destdir) {
#    mkdir $destdir or die "$destdir: $!";
#  }
  my $fname = catfile($srcdir, "qrcodes.html");
  open(my $fh, "<", $fname) or die "$fname: $!";
  my $s = join("", <$fh>);
  close($fh);
  my $title;
  if ($s =~ /\<body\>/i) {
    $s = $';
    if ($s =~ /\<h2\>([^\<]*)\<\/h2\>/) {
      $title = $1;
    }
  }
  die "$fname: Parse Error." if ! defined $title;
  $title =~ s/\s*- QR codes\s*$//is;

  if ($MODE eq "separate") {
    cpdf_open($cpdf_front, $destdir . "." . $EXT, Title => $title);
  }

  opendir(my $dh, $srcdir) or die "$srcdir: $!";
  my @l = sort (grep {/^qr[01-9]+\./} (readdir $dh));
  if ($MODE eq "twosides") {
    @l = (@l[0 .. (ceil(@l / 2) - 1)], reverse(@l[ceil(@l / 2) .. $#l]));
  }
  foreach my $f (@l) {
    next if $f !~ /^qr([01-9]+)\./;
    my $num = int($1);
    my $fname = catfile($srcdir, $f);
    my $src = Image::Magick->new;
    $src->Read($fname);
    $src->Flatten();
    $src->Draw(primitive => 'rectangle',
	       points=>"0, 0 64,16",
	       fill=>'white');
    my $w = ($PAPER_W - $QR_MARGIN) * $DPI;
    my $h = $src->Get('height') * $w / $src->Get('width');
    $w = int($w);
    $h = int($h);
    $src->Resize(width => $w, height => $h, antialias => 0);
    my $img = Image::Magick->new(size => int($PAPER_W * $DPI) . "x" . int($PAPER_H * $DPI));
    $img->ReadImage("xc:white");
    my $x = int(($PAPER_W * $DPI - $w) * 0.5);
    my $y = int(($PAPER_H * $DPI - $h) * 0.5);
    $x = '+' . $x if $x >= 0;
    $y = '+' . $y if $y >= 0;
    $img->Composite(image => $src, compose => 'atop', geometry=> $x . $y);
    $img->Annotate(
		   font      => 'c:/windows/fonts/arial.ttf',
		   pointsize => 8 * $PT_TO_INCH * $DPI,
		   fill      => 'black',
		   text      => $title,
		   x         => $QR_MARGIN * $DPI,
		   y         => $QR_MARGIN * $DPI,
		  );
    $img->Annotate(
		   font      => 'c:/windows/fonts/arial.ttf',
		   pointsize => 8 * $PT_TO_INCH * $DPI,
		   fill      => 'black',
		   text      => sprintf("%2d / %2d", $num, int(@l)),
		   align     => 'center',
		   x         => $PAPER_W * 0.5 * $DPI,
		   y         => $QR_MARGIN * $DPI + 16 * $PT_TO_INCH * $DPI,
		  );
    $img->Rotate(90);
    if ($MODE ne "twosides" || ($num - 1) < int(@l) / 2) {
      cpdf_shipout($cpdf_front, $img, $destdir);
    } else {
      cpdf_shipout($cpdf_back, $img, $destdir);
    }
  }
  closedir($dh);

  if ($MODE eq "twosides" && (@l % 2)) {
#    my $img = Image::Magick->new(size => $CARD_HEIGHT . "x" . $CARD_WIDTH);
#    $img->ReadImage('xc:white');
#    cpdf_shipout($cpdf_back, $img, $destdir);
    my $cpdf = $cpdf_back;
    $cpdf->{page_items}++;
    if ($cpdf->{page_items} >= $COLS * $ROWS) {
      $cpdf->{page_num}++;
      print "$cpdf->{last_target} $cpdf->{page_num}\n";
      $cpdf->{page} = undef;
    }
  }

  if ($MODE eq "separate") {
    cpdf_close($cpdf_front);
  }
}

if ($MODE eq "twosides") {
  cpdf_close($cpdf_back);
}
if ($MODE ne "separate") {
  cpdf_close($cpdf_front);
}
