#!/usr/bin/perl
#require 5.008_003;
our $VERSION = "0.02"; #Time-stamp: <2014-06-08T04:26:59Z>
our $DEBUG = 1;

use strict;
use warnings;
use utf8; # Japanese, English

use Getopt::Long;
use Pod::Usage;
$ENV{"PERLDOC"} = "" if ! exists $ENV{"PERLDOC"};
$ENV{"PERLDOC"} .= " " if $ENV{"PERLDOC"} ne "";
$ENV{"PERLDOC"} .= "-wcenter:'Probability Model'";

our $TRIALS = 10000;
our $DOORS = 4;
#our $MODE = "always";
our $MODE = "slm";

my $Prize = 0;
my $Valid = 0;


Getopt::Long::Configure("bundling", "auto_version");
GetOptions(
	   "trial=i" => \$TRIALS,
	   "door=i" => \$DOORS,
	   "slm" => sub {$MODE = "slm"},
	   "always" => sub {$MODE = "always"},
	   "man" => sub {pod2usage(-verbose => 2)},
	   "h|?" => sub {pod2usage(-verbose => 0, -output=>\*STDOUT, 
				   -exitval => 1)},
	   "help" => sub {pod2usage(1)},
	   ) or pod2usage(-verbose => 0);


sub closing {
  my (@door) = @_;
  my $r = 0;
  foreach my $d (@door) {
    $r++ if $d != -1;
  }
  return $r;
}

sub monty {
  my ($cur_choice, $closed, $doors) = @_;
  return undef if $closed < 3;
  my $num = $closed - 2;
  $num = $closed - 1 if $doors->[$cur_choice] == 1;
  my $o = int(rand($num));
  my $j = 0;
  for (my $i = 0; $i < @$doors; $i++) {
    next if $cur_choice == $i;
    my $d = $doors->[$i];
    next if $d != 0;
    if ($j != $o) {
      $j++;
      next;
    }
    $doors->[$i] = -1;
    return $i;
  }
  die "Unreacheable!";
}

sub change {
  my ($cur_choice, $closed, $doors) = @_;
  return undef if $closed < 2;
  my $o = int(rand($closed - 1));
  my $j = 0;
  for (my $i = 0; $i < @$doors; $i++) {
    next if $cur_choice == $i;
    my $d = $doors->[$i];
    next if $d == -1;
    if ($j != $o) {
      $j++;
      next;
    }
    return $i;
  }
  die "Unreacheable!";
}


for (my $i = 0; $i < $TRIALS; $i++) {
  my $prize = int(rand($DOORS));
  my @door = (0) x $DOORS;
  $door[$prize] = 1; # monty が開けてはずれたのは $door[] = -1 とする。
#  my $first_choice = int(rand($DOORS));
  my $first_choice = 0; # 原著通りにするなら、固定しても同じ。
  my $choice = $first_choice;
  while (1) {
    my $closed = closing(@door);
    last if $closed <= 2;
    my $monty = monty($choice, $closed, \@door);
    $closed--;

    if ($MODE eq "slm") {
      if ($closed == 2) {
	$choice = change($first_choice, $closed, \@door);
      }
    } elsif ($MODE eq "always") {
      $choice = change($choice, $closed, \@door);
    }
  }
  $Valid++;
  if ($door[$choice] == 1) {
    $Prize++;
  }
}

printf("Valid Trials = %d (/ %d)\n", $Valid, $TRIALS) if $Valid != $TRIALS;
printf("Prize = %d (%g)\n", $Prize, $Prize / $Valid) if $Valid > 0;
printf("Prize = 0\n") if $Valid == 0;


__END__

=pod

=encoding utf8

=head1	NAME

progressive_monty - 漸進的モンティ・ホール問題

=head1	SYNOPSIS

perl B<progressive_monty.pl> --door=NUM [--slm]

=head1	Options

=over 8

=item B<--help>

show help message about options.

=item B<--man>

show man page.

=item B<--version>

show version infomation.

=item B<--trial>

試行数。

=item B<--door>

ドアの数。

=item B<--slm>

SLM 戦略を使う。

=item B<--always>

常に変える戦略を使う。

=back

=head1	DESCRIPTION

ローゼンハウス 著『モンティ・ホール問題』の漸進モンティを試してみる。

=head1	AUTHOR

JRF

=head1	WEB ARTICLE

《JRF のひとこと：うわっ、やられた！何度か言ってた変形「三囚人問題」、変形モンティ・ホール問題とし...》
http://jrf.cocolog-nifty.com/statuses/2014/06/post-e55d.html

=head1	COPYRIGHT

Public Domain.

=cut
