#!/usr/bin/perl
#require 5.008_003;
our $VERSION = "0.02"; #Time-stamp: <2014-06-08T03:57:20Z>
our $DEBUG = 1;

use strict;
use warnings;
use utf8; # Japanese, English

use Getopt::Long;
use Pod::Usage;
$ENV{"PERLDOC"} = "" if ! exists $ENV{"PERLDOC"};
$ENV{"PERLDOC"} .= " " if $ENV{"PERLDOC"} ne "";
$ENV{"PERLDOC"} .= "-wcenter:'Probability Model'";

our $TRIALS = 10000;
#our $TRIALS = 100000;
our $DOORS = 5;
our $MODE = "1";
our $TACTIC = "random";
#our $TACTIC = "always";
#our $TACTIC = "slm";
our $NEED_PRIZE = 0;

my $Valid = 0;
my @Prize = (0) x $DOORS;


Getopt::Long::Configure("bundling", "auto_version");
GetOptions(
	   "trial=i" => \$TRIALS,
#	   "door=i" => \$DOORS,
	   "mode=s" => \$MODE,
	   "slm" => sub {$TACTIC = "slm"},
	   "always" => sub {$TACTIC = "always"},
	   "random-choice" => sub {$TACTIC = "random"},
	   "prize" => sub {$NEED_PRIZE= 1},
	   "man" => sub {pod2usage(-verbose => 2)},
	   "h|?" => sub {pod2usage(-verbose => 0, -output=>\*STDOUT, 
				   -exitval => 1)},
	   "help" => sub {pod2usage(1)},
	   ) or pod2usage(-verbose => 0);


sub closing {
  my (@door) = @_;
  my $r = 0;
  foreach my $d (@door) {
    $r++ if $d != -1;
  }
  return $r;
}

sub monty {
  my ($cur_choice, $closed, $doors) = @_;
  return undef if $closed < 3;
  my $o = int(rand($closed - 2));
  my $j = 0;
  for (my $i = 0; $i < @$doors; $i++) {
    next if $cur_choice == $i;
    my $d = $doors->[$i];
    next if $d != 0;
    if ($j != $o) {
      $j++;
      next;
    }
    $doors->[$i] = -1;
    return $i;
  }
  die "Unreacheable!";
}

sub change {
  my ($cur_choice, $closed, $doors) = @_;
  return undef if $closed < 2;
  my $o = int(rand($closed - 1));
  my $j = 0;
  for (my $i = 0; $i < @$doors; $i++) {
    next if $cur_choice == $i;
    my $d = $doors->[$i];
    next if $d == -1;
    if ($j != $o) {
      $j++;
      next;
    }
    return $i;
  }
  die "Unreacheable!";
}

sub choose {
  my ($cur_choice, $closed, $doors) = @_;
  return undef if $closed < 2;
  my $o = int(rand($closed - 0));
  my $j = 0;
  for (my $i = 0; $i < @$doors; $i++) {
#    next if $cur_choice == $i;
    my $d = $doors->[$i];
    next if $d == -1;
    if ($j != $o) {
      $j++;
      next;
    }
    return $i;
  }
  die "Unreacheable!";
}


for (my $i = 0; $i < $TRIALS; $i++) {
  my $prize = int(rand($DOORS));
  my @door = (0) x $DOORS;
  $door[$prize] = 1; # monty が開けてはずれたのは $door[] = -1 とする。
  my $first_choice = 0; # 原著通りに固定。
  my $choice = $first_choice;
  my $valid = 1;
  my $count = 0;
  my @monty;
  my @choice;
  my $reason = "why?";
  while (1) {
    push(@choice, $choice);
    my $closed = closing(@door);
    if ($MODE eq "1" && $closed == 5) {
      $count = 1;
    }
    if ($closed <= 2) {
      $valid = 1;
      last;
    }
    my $monty = monty($choice, $closed, \@door);
    push(@monty, $monty);
    $closed--;
    if ($MODE ne "1" && $closed == 4) {
      if ($monty != 1) {
	$valid = 0;
	$reason = "monty != 1";
	last;
      }
    }
    if ($MODE eq "2" && $closed == 4 && $monty == 1) {
      $count = 1;
    }

    if ($MODE eq "3b" && $closed == 3) {
      if ($monty != 0) {
	$valid = 0;
	last;
      } else {
	$count = 1;
      }
    }

    if ($MODE eq "3a" && $closed == 3) {
      if ($monty != 4) {
	$valid = 0;
	$reason = "monty != 4";
	last;
      } else {
	$count = 1;
      }
    }

    if ($MODE ne "1" && $closed == 4) {
      if ($door[2] == -1) {
	$reason = "door[2] == -1";
	$valid = 0;
	last;
      }
      $choice = 2;
    } else {
      if ($TACTIC eq "slm") {
	if ($closed == 2) {
	  $choice = change($choice, $closed, \@door);
	}
      } elsif ($TACTIC eq "always") {
	$choice = change($choice, $closed, \@door);
      } else {
	$choice = choose($choice, $closed, \@door);
      }
    }
  }

  warn "$i:prize==2 valid==$valid reason==\"$reason\" choice==$choice\n" if $DEBUG && $prize == 2 && $MODE eq "3a";
  if ($NEED_PRIZE) {
    if ($door[$choice] != 1) {
      $valid = 0;
    }
  }

  if ($valid) {
    if ($count) {
      $Prize[$prize]++;
    }
    print "m=" . join(",", @monty) . " " if $DEBUG;
    print "c=" . join(",", @choice) . " p=$prize\n" if $DEBUG;
    $Valid++;
  }
}

printf("Valid Trials = %d (/ %d)\n", $Valid, $TRIALS) if $Valid != $TRIALS;
printf("Prizes/Valid = (%g, %g, %g, %g, %g)\n", $Prize[0] / $Valid, $Prize[1] / $Valid, $Prize[2] / $Valid, $Prize[3] / $Valid, $Prize[4] / $Valid) if $Valid > 0;


__END__

=pod

=encoding utf8

=head1	NAME

progressive_monty2 - 漸進的モンティ・ホール問題 事例研究

=head1	SYNOPSIS

perl B<progressive_monty2.pl> --mode=[1|2|3a|3b]

=head1	Options

=over 8

=item B<--help>

show help message about options.

=item B<--man>

show man page.

=item B<--version>

show version infomation.

=item B<--trial>

試行数。

=item B<--mode>=[1|2|3a|3b]

動作モードの指定。

=item B<--slm>

SLM 戦略を使う。

=item B<--always>

常に変える戦略を使う。

=item B<--random-choice>

戦略として、常に変えるが、今指定している場所でもいい。

=item B<--prize>

商品が得られたときのみカウント。

=back

=head1	DESCRIPTION

ローゼンハウス 著『モンティ・ホール問題』の漸進モンティの扉５枚の場合を
試してみる。

原著通りにならず、どこかに不具合があるらしい。


This is a try of an example of J. Rosenhouse's "The Monty Hall
Problem".

Doesn't work well.  This program has bugs or logical errors, perhaps.

=head1	AUTHOR

JRF

=head1	WEB ARTICLE

《JRF のひとこと：うわっ、やられた！何度か言ってた変形「三囚人問題」、変形モンティ・ホール問題とし...》
http://jrf.cocolog-nifty.com/statuses/2014/06/post-e55d.html

=head1	COPYRIGHT

Public Domain.

=cut
