#!/usr/bin/perl
#require 5.008_003;
{ package main;
  our $VERSION = "0.07"; #Time-stamp: <2014-06-06T06:22:07Z>
  our $DEBUG = 1;
}

## License:
##
##   I in a provincial state made this program intended to be public-domain. 
##   But it might be better for you like me to treat this program as such 
##   under the BSD-License or under the Artistic License.
##
##   Within three months after the release of this program, I
##   especially admit responsibility of effort for rational request of
##   correction to this program.
##
## Author's Link:
##
##   http://jrf.cocolog-nifty.com/software/
##   (The page is written in Japanese.)
##

use strict;
use warnings;
no autovivification qw(strict warn fetch exists delete store);
use utf8; # Japanese English

## See below to know the object-oriented techniques of this program:
##
##   《Perl でオブジェクト指向 Ｃ＋＋風 その３ ローカル関数》
##   http://jrf.cocolog-nifty.com/software/2011/01/post-1.html
##

{
  package Resource;

  our @RESOURCE_PATH = (".", "./trl", "./perl_class",
			"$ENV{HOME}/.trl_init",
			"/usr/share/misc/jrf_semaphore", # Default.
			"/usr/share/jrf_semaphore", # Alternative.
		       );

  our $CONSOLE_ENCODING = "utf8";
  our $FILENAME_ENCODING = "utf8";
  our $SEMAPHORE_ASM = "jrf_semaphore.asm";
  our $SEMAPHORE_NES_ROM = "jrf_semaphore.nes";

  BEGIN {
    $INC{"JRF/Resource.pm"} = $0;
    {
      package JRF::Resource;
      use base qw(Exporter);
      our @EXPORT = qw(locate_file_resource require_class provide_class);
      no strict; no warnings;
      *{locate_file_resource} = \&Resource::locate_file_resource;
      *{require_class} = \&Resource::require_class;
      *{provide_class} = \&Resource::provide_class;
    }
  }
}

BEGIN {
  $INC{"JRF/Utils.pm"} = $0;
  {
    package JRF::Utils;
    use base qw(Exporter);
    our @EXPORT_OK = qw(encode_con decode_con encode_fn decode_fn
			encode_u decode_u parse_int unpack_bits pack_bits
			xd dump);
    our %EXPORT_TAGS = (all => \@EXPORT_OK, debug => [qw(xd dump)]);
    __PACKAGE__->import(@EXPORT_OK);
  }
}
BEGIN {
  $INC{"JRF/Utils/HashFreezer.pm"} = $0;
  $INC{"JRF/Utils/Hooker.pm"} = $0;
}

BEGIN {
  my @mypackage = qw(
		      Main
		      JRF::MyOO
		      JRF::Dummy
		      JRF::Puppet
		      JRF::Figure
		      Naggy
		      Naggy::Translit
		      Naggy::Translit::Function
		      Naggy::Backend::Translit
		      Naggy::Backend::CommandInterface
		      NES::ChrPage
		      NES::RomTool
		      JRF::FlagSemaphore
		      JRF::FlagSemaphore::Sailor
		      GifFont
		      GifFont::GIF
		      GifFont::JRFSemaphoreRom
		      JRF::FlagSemaphore::NgbCommand
		      JRF::FlagSemaphore::Command
		   );

  foreach my $p (@mypackage) {
    eval <<"EOM";
    {
      package $p;
      use JRF::Resource;
      use JRF::Utils qw(:all);
    }
EOM
    die $@ if $@;
  }

  my @myfunction = qw(
		       JRF::MyOO::new_scalar
                    );
  {
    no strict; ## 型グロブの警告を停止。Stop warnings of type glob.

    foreach my $p (@mypackage) {
      for my $f (@myfunction) {
        my @f = split("::", $f);
        *{$p . "::" . $f[$#f]} = \&{$f};
      }
    }
  }

  my @myvariable = qw(
		       main::VERSION
		       main::DEBUG
                    );
  {
    no strict; ## 型グロブの警告を停止。Stop warnings of type glob.

    foreach my $p (@mypackage) {
      for my $f (@myvariable) {
        my @f = split("::", $f);
        *{$p . "::" . $f[$#f]} = \*{$f};
      }
    }
  }
}


{
  package Resource;
  use GD::simple;
  use File::Spec::Functions qw(catfile file_name_is_absolute rel2abs
			       splitpath splitdir no_upwards);
  use JRF::Utils qw(encode_fn);

  our %PROVIDED_CLASS;
  our %COLOR_NAME_LC;

  {
    package JRF::Resource;
    use base qw(Exporter);
    our @EXPORT = qw(locate_file_resource require_class provide_class);
    no strict; no warnings;
    *{locate_file_resource} = \&Resource::locate_file_resource;
    *{require_class} = \&Resource::require_class;
    *{provide_class} = \&Resource::provide_class;
  }

  sub is_subpath {
    my ($a, $b, $child_only) = @_;
    my ($av, $ad) = splitpath($a, 1);
    my ($bv, $bd) = splitpath($b, 1);
    my @ad = ($av, splitdir($ad));
    my @bd = ($bv, splitdir($bd));
    return 0 if defined $child_only && $child_only && @ad != @bd + 1;
    for (my $i = 0; $i < @bd; $i++) {
      return 0 if $i >= @ad || $bd[$i] ne $ad[$i];
    }
    return 1;
  }

  sub is_last_path {
    my ($a, $b) = @_;
    return 0 if ! (length($b) >= length($a) && substr($b, -length($a)) eq $a);
    my ($av, $ad) = splitpath($a, 1);
    my ($bv, $bd) = splitpath($b, 1);
    return 0 if $av ne ""  && $av ne $bv;
    my @ad = reverse splitdir($ad);
    my @bd = reverse splitdir($bd);
    for  (my $i = 0; $i < @ad; $i++) {
      return 0 if $i >= @bd || $ad[$i] ne $bd[$i];
    }
    return 1;
  }

  sub locate_file_resource {
    my ($f) = @_;

    return undef if ! defined $f || $f eq "";
    my ($v, $d, $b) = splitpath($f);
    if (defined $d && ! file_name_is_absolute($f)) {
      my @d = splitdir($d);
      return undef if no_upwards(@d) != @d;
    }

    foreach my $d (@Resource::RESOURCE_PATH) {
      my $p;
      if (file_name_is_absolute($f)) {
	if ($d eq "") {
	  $p = $f;
	} elsif (is_subpath($f, rel2abs($d))) {
	  $p = $f;
	}
      } else {
	if (is_last_path($f, $d)) {
	  $p = rel2abs($d);
	} elsif ($d ne "") {
	  $p = rel2abs(catfile($d, $f));
	}
      }
      if (defined $p) {
	my $e = encode_fn($p);
	return $p if -f $e || -d $e;
      }
    }
    return undef;
  }

  sub require_class {
    my ($class, @opt) = @_;
    return if exists $Resource::PROVIDED_CLASS{$class};
    my $l = $class;
    $l=~ s/\:\:/\//g;
    $l = locate_file_resource($l . ".poo") 
      || locate_file_resource($l . ".plo")
      || locate_file_resource($l . ".pm")
	|| locate_file_resource($l . ".pl")
	  || $class;
    require $l;
  }

  sub provide_class {
    my ($class) = @_;
    my $l = $class;
    $Resource::PROVIDED_CLASS{$class} = $class;
  }

 ModuleInit:
  {
    binmode(STDOUT, ":" . $Resource::CONSOLE_ENCODING) if $main::DEBUG;
    binmode(STDERR, ":" . $Resource::CONSOLE_ENCODING);

    my $cn = GD::Simple->color_names();
    foreach my $k (keys %$cn) {
      my $v = $cn->{$k};
      $COLOR_NAME_LC{lc($k)} = $v;
    }
  }
}


{
  package JRF::MyOO;
  provide_class __PACKAGE__;
  ## 「俺様」流オブジェクト指向 (^^;のベースオブジェクト。
  ## The base object of the unbelievably profound
  ## Object-Oriented Programming way of mine. ;-)

  use Storable qw(dclone);
  use Carp;
  use JRF::Utils::HashFreezer;
  use JRF::Utils::Hooker;

  my %template = ("JRF::MyOO" => {cvar => {}});
  my %cvar = ("JRF::MyOO" => {});

  our $CVAR_DEBUG;

  sub extend_template {
    my $class = shift;
    my @hash = @_;
    if (exists $template{$class}) {
      $template{$class} = {%{$template{$class}}, @hash} if @hash;
    } else {
      $template{$class} =
        {(map {
                $_->extend_template() if ! exists $template{$_};
                %{$template{$_}};
              } (eval '@{' . $class . '::ISA}')),
         @hash
        };
    }
    return $template{$class};
  }

  sub extend_cvar {
    my $class = shift;
    my @hash = @_;
    if (exists $cvar{$class}) {
      $cvar{$class} = {%{$cvar{$class}}, @hash} if @hash;
    } else {
      $cvar{$class} =
        {(map {
                $_->extend_cvar() if ! exists $cvar{$_};
                %{$cvar{$_}};
              } (eval '@{' . $class . '::ISA}')),
         @hash
        };
    }
    return $cvar{$class};
  }

  sub get_template {
    my $class = shift;
    croak((ref $class) . "::get_template : requires class name not object.")
      if ref $class;
    return $class->extend_template();
  }

  sub get_cvar {
    my $class = shift;
    croak((ref $class) . "::get_cvar : requires class name not object.")
      if ref $class;
    return $class->extend_cvar();
  }

  sub clone {
    croak((ref $_[0]) . "::clone : forbidden.");
  }

  sub add_cvar_hook {
    my $self = shift;
    my ($name, $data) = @_;
    my $hooker = (tied %{$self->{cvar}})->[1];
    $hooker->{store_hook}->{$name} = []
      if ! exists $hooker->{store_hook}->{$name};
    push(@{$hooker->{store_hook}->{$name}}, $data);
  }

  sub remove_cvar_hook {
    my $self = shift;
    my ($name, $data) = @_;
    my $hooker = (tied %{$self->{cvar}})->[1];
    return undef if ! exists $hooker->{store_hook}->{$name};
    my $l = $hooker->{store_hook}->{$name};
    for (my $i = 0; $i < @$l; $i++) {
      if ($l->[$i] eq $data) {
	splice(@$l, $i, 1);
	return $data;
      }
    }
    return undef;
  }

  sub new {
    my $class = shift;
    my $obj;
    my $template = $class->get_template();
    my $cvar = $class->get_cvar();
    my $cdb = (defined $CVAR_DEBUG)? $CVAR_DEBUG : $DEBUG;
    if ($DEBUG) {
      $obj = {};
      tie %$obj, 'JRF::Utils::HashFreezer', (template => $template);
      %{$obj} = %{dclone($template)};
    } else {
      $obj = dclone($template);
    }
    if ($cdb) {
      my $cv = {};
      tie %$cv, 'JRF::Utils::Hooker', (template => $cvar,
				       store_hook => {}, fetch_hook => {});
      %{$cv} = %{$cvar};
      $obj->{cvar} = $cv;
    } else {
      %{$obj->{cvar}} = %{$cvar};
    }
    bless $obj, $class;
    return $obj;
  }

  my $_scalar_base;
  sub new_scalar {
    my ($self) = (!(ref $_[0]) || (ref $_[0])->isa(__PACKAGE__))?
      shift : undef;
    return dclone(\$_scalar_base);
  }
}


{
  package JRF::Utils::HashFreezer;
  use Tie::Hash;
  use Carp;
  use base qw(Tie::ExtraHash);

  sub TIEHASH {
    my $class = shift;
    return (bless [{}, {@_}], $class);
  }

  sub STORE {
    if (! exists $_[0][1]->{template}->{$_[1]}) {
      carp "The key '$_[1]' doesn't exist in the hash.\n";
    }
    $_[0][0]{$_[1]} = $_[2];
  }

  sub FETCH {
    if (! exists $_[0][1]->{template}->{$_[1]}) {
      carp "The key '$_[1]' doesn't exist in the hash.\n";
    }
    return $_[0][0]{$_[1]};
  }
}


{
  package JRF::Utils::Hooker;
  use JRF::Utils::HashFreezer;
  use Carp;
  use base qw(JRF::Utils::HashFreezer);

  sub STORE {
    my $self = shift;
    $self->SUPER::STORE(@_);
    unshift(@_, $self);
    if (exists $_[0][1]->{store_hook}->{$_[1]}) {
      my @hooks = @{$_[0][1]->{store_hook}->{$_[1]}};
      foreach my $h (@hooks) {
	next if ! ref $h;
	if ((ref $h) eq 'CODE') {
	  &{$h}($_[1], $_[2]);
	} elsif ((ref $h) eq 'ARRAY') {
	  my ($f, @opt) = @$h;
	  &{$f}(@opt, $_[1], $_[2]);
	} elsif (defined &{(ref $h) . "::process_command"}) {
	  $h->process_command($_[1], $_[2]);
	} else {
	  $h->{$_[1]} = $_[2];
	}
      }
    }
  }

  sub FETCH {
    if (exists $_[0][1]->{fetch_hook}->{$_[1]}) {
      my @hooks = @{$_[0][1]->{fetch_hook}->{$_[1]}};
      foreach my $h (@hooks) {
	next if ! ref $h;
	if ((ref $h) eq 'CODE') {
	  &{$h}($_[1]);
	} elsif ((ref $h) eq 'ARRAY') {
	  my ($f, @opt) = @$h;
	  &{$f}(@opt, $_[1]);
	} elsif (defined &{(ref $h) . "::process_command"}) {
	  $h->process_command($_[1]);
	}
      }
    }
    my $self = shift;
    $self->SUPER::FETCH(@_);
  }
}


{
  package JRF::Dummy;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use Storable qw(dclone);

  sub meme {
    # As getter (without arguments),
    # with class, return "meme" i.e. information needed for the class,
    # with object, return the meme of the object.
    # As setter or filter (with some arguments),
    # with class, filter arguments to rest meme needed for the class,
    # with object, set meme needed for the object.
    if (ref $_[0]) {
      my $proto = shift;
      if (@_) {
	my $head = (@_ % 2 == 1)? shift : undef;
	my %opt = @_;
	my %ref = (ref $proto)->meme(%$proto);
	my %r;
	foreach my $k (keys %opt) {
	  $proto->{$k} = $opt{$k} if ! exists $ref{$k};
	  $r{$k} = $opt{$k} if exists $ref{$k};
	}
	return %r;
      } else {
	my $self = $proto;
	my %ref = (ref $self)->meme(%$self);
	my %r;
	foreach my $k (keys %$self) {
	  if (! exists $ref{$k}) {
	    $r{$k} = $self->{$k};
	  }
	}
	return %r;
      }
    } else {
      my $class = shift;
      if (@_) {
	my $head = (@_ % 2 == 1)? shift : undef;
	my @tmp = $class->meme();
	my $thead = (@tmp % 2 == 1)? shift(@tmp) : undef;
	if (defined $thead && defined $head) {
	  unshift(@tmp, $head);
	  unshift(@tmp, $thead);
	}
	my %tmp = @tmp;
	my %opt = @_;
	my %r;
	foreach my $k (keys %opt) {
	  $r{$k} = $opt{$k} if exists $tmp{$k};
	}
	return %r;
      } else {
	my %r = %{$class->get_template()};
	delete $r{cvar};
	return %r;
      }
    }
  }

  sub clone {
    my $self = shift;
    my @meme = $self->meme();
    $self = bless dclone({(ref $self)->meme(%$self)}), (ref $self);
    $self->meme(@meme);
    return $self;
  }
}

{
  package JRF::Puppet;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use Storable qw(dclone);

  sub gene { return (); }

  sub clone {
    my $self = shift;
    my %gene = $self->gene();
    my $obj = (ref $self)->new(%gene);
    foreach my $k (keys %{(ref $self)->get_template()}) {
      if (! exists $gene{$k}) {
	if ((ref $self->{$k}) && defined &{(ref $self->{$k}) . "::clone"}) {
	  $obj->{$k} = $self->{$k}->clone();
	} else {
	  $obj->{$k} = dclone($self->{$k});
	}
      }
    }
    return $obj;
  }
}

{
  package JRF::Figure; # with brain, no requirement for spirit.
  use base qw(JRF::Puppet JRF::Dummy);
  provide_class __PACKAGE__;

  use Carp;

  $JRF::MyOO::CVAR_DEBUG = 1;

  __PACKAGE__->extend_cvar
    (
     PUBLIC_true => "1",  # <- an example garbage.
    );

  sub brain {} # set brain.
  sub super_brain {  # brain of super class.
    if (ref $_[0]) {
      my $self = shift;
      return undef;
    } else {
      my $class = shift;
      my ($brain) = @_;
      return undef;
    }
  }

  sub gene {
    if (ref $_[0]) {
      my $self = shift;
      return %$self;
    } else {
      my $class = shift;
      return %{$class->get_template()};
    }
  }

  sub meme {
    if (! ref $_[0]) {
      my $class = shift;
      if (@_) {
	my $odd = (@_ % 2 == 1);
	my $head = ($odd)? shift : undef;
	my @r = $class->SUPER::meme(@_);
	unshift(@r, $head) if $odd;
	return @r;
      }
      my @gene = $class->gene();
      my $odd = (@gene % 2 == 1);
      my $head = ($odd)? shift(@gene) : undef;
      unshift(@_, $head) if defined $head;
      my %opt = @gene;
      my %ref = $class->SUPER::meme();
      my %r;
      foreach my $k (keys %ref) {
	$r{$k} = $opt{$k} if exists $ref{$k};
      }
      my $cvar = $class->get_cvar();
      foreach my $k (keys %$cvar) {
	if ($k =~ /^PUBLIC_/) {
	  my $n = $&;
	  my $v = $cvar->{$k};
	  $r{$n} = $v if ! exists $r{$n};
	}
      }
      my @r = %r;
      unshift(@r, $head) if $odd;
      return @r;
    } else {
      my $dummy = shift;
      return $dummy->SUPER::meme() if ! @_;
      my $head = (@_ % 2 == 1)? shift(@_) : undef;
      $dummy->brain($head);
      return $dummy->SUPER::meme(@_);
    }
  }

  sub clone {
    my $self = shift;
    my @meme = $self->meme();
    my @gene = $self->gene();
    my $obj = (ref $self)->new((ref $self)->meme(@gene));
    $obj->meme(@meme);
    return $obj;
  }

  sub add_cvar_hook {
    my $self = shift;
    my ($name, @rest) = @_;
    if ($name !~ /^PUBLIC_/) {
      if ($DEBUG) {
	carp "add_cvar_hook: is not allowed for $name.";
      } else {
	croak "add_cvar_hook: is not allowed for $name.";
      }
    }
    $self->SUPER::add_cvar_hook($name, @rest);
  }

  sub remove_cvar_hook {
    my $self = shift;
    my ($name, @rest) = @_;
    if ($name !~ /^PUBLIC_/) {
      if ($DEBUG) {
	carp "remove_cvar_hook: is not allowed for $name.";
      } else {
	croak "remove_cvar_hook: is not allowed for $name.";
      }
    }
    $self->SUPER::remove_cvar_hook($name, @rest);
  }

  sub new {
    my $class = shift;
    my $brain = (@_ % 2 == 1)? shift : undef;
    my %opt = @_;
    my $cdb = $JRF::MyOO::CVAR_DEBUG;
    $JRF::MyOO::CVAR_DEBUG = 1;

    my $sbrain;
    $sbrain = $class->super_brain($brain) if defined $brain;
    unshift(@_, $sbrain) if defined $sbrain;
    my $obj = $class->SUPER::new(@_);
    my %smeme = (map {$_->meme()} (eval '@{' . $class . '::ISA}'));
    my %meme = $class->meme();
    foreach my $k (keys %opt) {
      if (exists $meme{$k} && ! exists $smeme{$k}) {
	$obj->{$k} = $opt{$k};
      }
    }
    $obj->meme($brain);

    $JRF::MyOO::CVAR_DEBUG = $cdb;
    return $obj;
  }
}


{
  package Naggy;

  sub escape_string {
    my ($src) = @_;
    return undef if ! defined $src;
    return "\\0" if $src eq "";
    $src =~ s([\x00-\x20\x7F\\\#]){
      sprintf("\\x%02X", ord($&));
    }gsex;
    return $src;
  }

  sub unescape_string {
    my ($src) = @_;
    return undef if ! defined $src;
    $src =~ s(\\([0nt]|[^01-9A-Za-z]|x[01-9a-fA-F][01-9a-fA-F]|u\{[01-9a-fA-F]+\}|u\([01-9a-fA-F]+\))){
      if ($1 eq "0") {
	"";
      } elsif (substr($&, 1, 1) eq "x") {
	chr(hex(substr($&, 2)));
      } elsif (substr($&, 1, 1) eq "u") {
	chr(hex(substr($&, 3, -1)));
      } elsif ($1 eq "n") {
	"\n";
      } elsif ($1 eq "t") {
	"\t";
      } else {
	$1;
      }
    }gsex;
    return $src;
  }

  sub regular_string {
    my ($src) = @_;
    $src =~ s/\\\\/\\u\{5C\}/gs;
    $src =~ s(\\([^01-9A-Za-z])){sprintf("\\u{%X}", ord($1))}gsex;
    return $src;
  }

  sub is_true {
    my ($s) = @_;
    return defined $s && $s ne "" && $s !~ /^(?:false|0+)$/si;
  }

  sub next_line {
    my ($l) = @_;
    my $r;
    my $done = 0;
    while (! $done && @$l) {
      my $s = shift(@$l);
      $s =~ s/\x0a$//s;
      $s =~ s/\x0d$//s;
      $s = Naggy::regular_string($s);
      $done = ($s !~ s/\\$//);
      $s =~ s/\#.*$//;
      if (defined $r) {
	$r .= $s;
      } else {
	$r = $s;
      }
    }
    return $r;
  }
}


{
  package Naggy::Translit;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use Encode qw();

  __PACKAGE__->extend_template
    (
     type => "translit",
     case_sensitive => 1,
     max_length_table => {},
     table => {},
     ideo => {},
     completion_beginning => undef,
     auto_insert_first => undef,
     auto_insert_last => undef,
     recursive => 0,
    );

  sub load_from_string {
    my $self = (ref $_[0])? shift : __PACKAGE__->new();
    my ($src, %opt) = @_;
    $self->{case_sensitive} = $opt{case_sensitive}
      if exists $opt{case_sensitive};

    my $intable = 0;
    my @line = split(/\n/, $src);
    while (@line) {
      my $s = Naggy::next_line(\@line);
      $s =~ s/^\s+//;
      $s =~ s/\s+$//;

      if (! $intable && $s =~ s/^\.\s*//) {
	if ($s =~ /^case[- ]sensitive$/i 
	    || ($s =~ /^case[- ]sensitive\s+(\S.*)$/i
		&& Naggy::is_true($1))) {
	  $self->{case_sensitive} = 1;
	} elsif ($s =~ /^case[- ]insensitive$/i 
		 || ($s =~ /^case[- ]sensitive\s+(\S.*)$/i
		     && ! Naggy::is_true($1))) {
	  $self->{case_sensitive} = 0;
	} elsif ($s =~ /^completion[- ]beginning\s+/i) {
	  my @l = map {Naggy::unescape_string($_)} split(/\s+/, $');
	  my $tmp = $self->{completion_beginning};
	  foreach my $w (@l) {
	    if ($self->{case_sensitive}) {
	      if (defined $tmp) {
		$tmp = qr/$tmp|\Q$w\E/;
	      } else {
		$tmp = qr/\Q$w\E/;
	      }
	    } else {
	      if (defined $tmp) {
		$tmp = qr/$tmp|\Q$w\E/i;
	      } else {
		$tmp = qr/\Q$w\E/i;
	      }
	    }
	  }
	  $self->{completion_beginning} = $tmp;
	} elsif ($s =~ /^auto[- ]insert[- ]first\s+(\S+)$/i) {
	  $self->{auto_insert_first} = Naggy::unescape_string($1);
	} elsif ($s =~ /^auto[- ]insert[- ]last\s+(\S+)$/i) {
	  $self->{auto_insert_last} = Naggy::unescape_string($1);
	} elsif ($s =~ /^recursive(?:\s+(?:true|1))?$/i) {
	  $self->{recursive} = 1;
	}
	$s = "";
      }

      next if $s eq "";

      $intable = 1;

      my ($from, $to, $ideo) = split(/\s+/, $s);
      $from = Naggy::unescape_string($from);
      $to = Naggy::unescape_string($to) if defined $to;
      $to = $from if ! defined $to;
      $ideo = Naggy::unescape_string($ideo) if defined $ideo;
      $from = uc($from) if ! $self->{case_sensitive};
      my $c = substr($from, 0, 1);
      if (! exists $self->{max_length_table}->{$c}
	  || length($from) > $self->{max_length_table}->{$c}) {
	$self->{max_length_table}->{$c} = length($from);
      }
      $self->{table}->{$from} = $to;
      $self->{ideo}->{$from} = $ideo if defined $ideo;
    }

    return $self;
  }

  sub inverted_copy {
    my $src = shift;
    my $dest = __PACKAGE__->new();
    my %opt = (pref => sub {
		 my ($a, $b) = @_;
		 return length($a) <=> length($b);
	       },
	       @_);
    foreach my $k (keys %{__PACKAGE__->get_template()}) {
      $dest->{$k} = $opt{$k} if exists $opt{$k};
    }
    foreach my $sfrom (keys %{$src->{table}}) {
      my $sto = $src->{table}->{$sfrom};
      my $sideo;
      $sideo = $src->{ideo}->{$sfrom} if exists $src->{ideo}->{$sfrom};
      my $dfrom;
      my $dto;
      $dfrom = $sto;
      $dto = $sfrom;
      $dfrom = uc($dfrom) if ! $dest->{case_sensitive};
      if (! exists $dest->{table}->{$dfrom}
	  || &{$opt{pref}}($dto, $dest->{table}->{$dfrom}) > 0) {
	my $c = substr($dfrom, 0, 1);
	if (! exists $dest->{max_length_table}->{$c}
	    || length($dfrom) > $dest->{max_length_table}->{$c}) {
	  $dest->{max_length_table}->{$c} = length($dfrom);
	}
	$dest->{table}->{$dfrom} = $dto;
      }
      next if ! defined $sideo;
      $dfrom = $sideo;
      $dto = $sfrom;
      $dfrom = uc($dfrom) if ! $dest->{case_sensitive};
      if (! exists $dest->{table}->{$dfrom}
	  || &{$opt{pref}}($dto, $dest->{table}->{$dfrom}) > 0) {
	my $c = substr($dfrom, 0, 1);
	if (! exists $dest->{max_length_table}->{$c}
	    || length($dfrom) > $dest->{max_length_table}->{$c}) {
	  $dest->{max_length_table}->{$c} = length($dfrom);
	}
	$dest->{table}->{$dfrom} = $dto;
      }
    }
    return $dest;
  }

  sub put {
    my $self = shift;
    my ($from, $to) = @_;
    $from = uc($from) if $self->{case_sensitive};
    if (! exists $self->{table}->{$from}) {
      my $c = substr($from, 0, 1);
      if (! exists $self->{max_length_table}->{$c}
	  || length($from) > $self->{max_length_table}->{$c}) {
	$self->{max_length_table}->{$c} = length($from);
      }
    }
    $self->{table}->{$from} = $to;
  }

  sub remove {
    my $self = shift;
    my ($key) = @_;
    $key = uc($key) if $self->{case_sensitive};
    return undef if ! exists $self->{table}->{$key};
    my $r = delete $self->{table}->{$key};
    my $c = substr($key, 0, 1);
    return $r if length($key) < $self->{max_length_table}->{$c};
    my @l = grep {substr($_, 0, 1) eq $c} (keys %{$self->{table}});
    if (! @l) {
      delete $self->{max_length_table}->{$c};
    } else {
      my $max = 0;
      foreach my $k (@l) {
	$max = length($k) if $max < length($k);
      }
      $self->{max_length_table}->{$c} = $max;
    }
    return $r;
  }

  sub load_trl {
    my $self = (ref $_[0])? shift : undef;
    my ($f) = @_;
    my $case_sensitive = 1;
    my $enc = Encode::find_encoding('utf8') or die "# error ";
    open(my $fh,  "<", encode_fn($f)) or return undef;
    binmode($fh, ":raw");
    my $src = join("", <$fh>);
    my @line = split(/\n/, $src);
    close($fh);
    while (@line) {
      $_ = Naggy::next_line(\@line);
      $_ = $enc->decode($_);
      if (/^\s*\./ || /^\s*$/) {
	if (/^\s*\.\s*encoding\s+(\S+)\s*$/i) {
	  $enc = Encode::find_encoding($1);
	  $src = $enc->decode($src);
	  last;
	}
      } else {
	last;
      }
    }
    if (defined $self) {
      return $self->load_from_string($src, case_sensitive => $case_sensitive);
    } else {
      return load_from_string($src, case_sensitive => $case_sensitive);
    }
  }

  sub load_wnd {
    my $self = (ref $_[0])? shift : undef;
    my ($f) = @_;
    my $src = "";
    open(my $fh, "<", encode_fn($f)) or return undef;
    binmode($fh, ":encoding(cp932):crlf");
    while (<$fh>) {
      chomp;
      if (/\t/) {
	my ($from, $to) = split(/\t/, $_);
	$src .= Naggy::escape_string($from) . "\t"
	  . Naggy::escape_string($to) . "\n";
      }
    }
    close($fh);
    if (defined $self) {
      return $self->load_from_string($src, case_sensitive => 0);
    } else {
      return load_from_string($src, case_sensitive => 0);
    }
  }

  sub load_file {
    my $f = (ref $_[0])? $_[1] : $_[0];

    if ($f =~ /\.wnd$/i) {
      return load_wnd(@_);
    } else {
      return load_trl(@_);
    }
  }

  sub translit {
    my $self = shift;
    my ($src) = @_;
    my $dest = "";
    if (defined $self->{auto_insert_first}) {
      $src = $self->{auto_insert_first} . $src;
    }
    if (defined $self->{auto_insert_last}) {
      $src .= $self->{auto_insert_last};
    }

    while (1) {
      my $prev = $src;
      $dest = "";

      while (length($src) > 0) {
	my $c = substr($src, 0, 1);
	$c = uc($c) if ! $self->{case_sensitive};
	if (! exists $self->{max_length_table}->{$c}) {
	  $dest = $dest . $c;
	  $src = substr($src, 1);
	} else {
	  my $done = 0;
	  for (my $len = $self->{max_length_table}->{$c}; $len > 0; $len--) {
	    next if length($src) < $len;
	    my $from = substr($src, 0, $len);
	    $from = uc($from) if ! $self->{case_sensitive};
	    if (exists $self->{table}->{$from}) {
	      $dest = $dest . $self->{table}->{$from};
	      $src = substr($src, $len);
	      $done = 1;
	      last;
	    }
	  }
	  if (! $done) {
	    $dest = $dest . $c;
	    $src = substr($src, 1);
	  }
	}
      }

      last if ! $self->{recursive} || $dest eq $prev;
      $src = $dest;
    }

    return $dest;
  }
}

{
  package Naggy::Translit::Function;
  use base qw(Naggy::Translit);
  provide_class __PACKAGE__;

  use Carp;

  __PACKAGE__->extend_template
    (
     function => undef,
    );

  sub inverted_copy {
    croak "Naggy::Translit::Function cannot be inverted!";
  }

  sub new {
    my $class = shift;
    my $obj =  $class->SUPER::new(@_);
    my ($f) = @_;
    $obj->{function} = $f;
    return $obj;
  }

  sub translit {
    my $self = shift;
    return &{$self->{function}}(@_);
  }
}

{
  package Naggy::Backend::Translit;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use Encode qw();

  __PACKAGE__->extend_template
    (
     table => {},
    );

#  my $DEFAULT_ALPHA_HWKATA_FILE
#    = locate_file_resource($Resource::DEFAULT_ALPHA_HWKATA_FILE);

  our %TRANSLIT_TABLE
    = (
       # abbrevs.
       # hw:	half width.
       # fw:	full width.
       # kata:	japanese katakana (default fw)
       # hira:	japanese hiragana (fw)
       # alpha:	ascii alphabets and some symbols (default hw)
       identical => [],
      );

  sub new {
    my $class = shift;
    my $obj =  $class->SUPER::new(@_);
    my %opt = @_;
    %{$obj->{table}} = %TRANSLIT_TABLE;
    if (exists $opt{PUNCTUATION_FULLWIDTH_PERIOD}
	&& $opt{PUNCTUATION_FULLWIDTH_PERIOD}) {
      $obj->{table}->{"alpha-hwkata"}
	= ["alpha-hwkata-raw", "punctuation-fwperiod"];
    } else {
      $obj->{table}->{"alpha-hwkata"}
	= ["alpha-hwkata-raw", "punctuation-japanese"];
    }
    return $obj;
  }

  sub translit {
    my $self = shift;
    my ($spec, $s) = @_;
    my $tbl = $self->{table};

    sub _naggy_translit {
      my ($tbl, $spec, $s, $visited) = @_;
      return undef if ! defined $s;
      return undef if exists $visited->{$spec};
      $visited->{$spec} = 1;

      if (ref $spec eq 'ARRAY') {
	foreach my $t (@{$spec}) {
	  $s = _naggy_translit($tbl, $t, $s, $visited);
	}
	return $s;
      } elsif (ref($spec)) {
	return $spec->translit($s);
      } else {
	foreach my $t (split(/\s+/, $spec)) {
	  return undef if ! exists $tbl->{$t};
	  $s = _naggy_translit($tbl, $tbl->{$t}, $s, $visited);
	}
	return $s;
      }
    }
    return _naggy_translit($tbl, $spec, $s, {});
  }

  sub rename {
    my $self = shift;
    my ($from, $to) = @_;
    my $tbl = $self->{table};

    return 0 if ! exists $tbl->{$from};

    sub _rename_in_string {
      my ($from, $to, $s) = @_;
      return join(" ", map {($_ eq $from)? $to : $_} split(/\s+/, $s));
    }

    sub _rename_in_array {
      my ($from, $to, @l) = @_;
      return map {
	if (ref($_) eq 'HASH') {
	  return $_;
	} elsif (ref($_) eq 'ARRAY') {
	  return [_rename_in_array($from, $to, @{$_})];
	} else {
	  return _rename_in_string($from, $to, $_);
	}
      } @l;
    }

    foreach my $key (keys %$tbl) {
      my $spec = $tbl->{$key};
      if (ref($spec) eq 'ARRAY') {
	$tbl->{$key} = [_rename_in_array($from, $to, @{$spec})];
      } elsif (ref($spec)) {
	# do none;
      } else {
	$tbl->{$key} = _rename_in_string($from, $to, $spec);
      }
    }

    $tbl->{$to} = $tbl->{$from};
    delete $tbl->{$from};
  }

  sub alpha_charmap {
    my $self = shift;
    my ($spec) = @_;
    my $tbl = $self->{table};

    sub _naggy_alpha_charmap {
      my ($tbl, $spec, $charmap, $visited) = @_;
      return undef if ! defined $charmap;
      return undef if exists $visited->{$spec};
      $visited->{$spec} = 1;

      if ((ref $_[0])->isa("Naggy::Translit")) {
	foreach my $key (keys %{$charmap}) {
	  $charmap->{$key} = $spec->translit($charmap->{$key});
	}
	foreach my $key (keys %{$spec->{table}}) {
	  next if $key !~ /^[\x21-\x7E]+$/;
	  next if exists $charmap->{$key};
	  my $ideo = $spec->{table}->{$key};
	  $ideo = $spec->{ideo}->{$key} if exists $spec->{ideo}->{$key};
	  $charmap->{$key} = $ideo;
	}
	return $charmap;
      } elsif (ref($spec) eq 'ARRAY') {
	foreach my $t (@{$spec}) {
	  $charmap = _naggy_alpha_charmap($tbl, $t, $charmap, $visited);
	  return undef if ! defined $charmap;
	}
	return $charmap;
      } else {
	foreach my $t (split(/\s+/, $spec)) {
	  return undef if ! exists $tbl->{$t};
	  $charmap = _naggy_alpha_charmap($tbl, $tbl->{$t}, $charmap, $visited);
	  return undef if ! defined $charmap;
	}
	return $charmap;
      }
    }

    return _naggy_alpha_charmap($tbl, $spec, {}, {});
  }

  sub completion_beginning {
    my $self = shift;
    my ($spec) = @_;
    my $tbl = $self->{table};

    sub _completion_beginning {
      my ($tbl, $spec, $re, $visited) = @_;
      return undef if exists $visited->{$spec};
      $visited->{$spec} = 1;

      if (ref($spec) eq 'ARRAY') {
	foreach my $t (@{$spec}) {
	  $re = _completion_beginning($tbl, $t, $re, $visited);
	}
	return $re;
      } elsif (ref($spec)) {
	my $tmp = $spec->{completion_beginning};
	if (defined $tmp) {
	  if (defined $re) {
	    return qr/$re|$tmp/;
	  } else {
	    return $tmp;
	  }
	} else {
	  return $re;
	}
      } else {
	foreach my $t (split(/\s+/, $spec)) {
	  return undef if ! exists $tbl->{$t};
	  $re = _completion_beginning($tbl, $tbl->{$t}, $re, $visited);
	}
	return $re;
      }
    }

    return _completion_beginning($tbl, $spec, undef, {});
  }

  sub complete {
    my $self = shift;
    my ($spec, $s) = @_;

    my $compbeg = $self->completion_beginning($spec);
    my $charmap = $self->alpha_charmap($spec);

    return undef if ! defined $charmap;

    my $prefix = "";

    if (defined $compbeg) {
      my $match = undef;
      while ($s =~ $compbeg) {
	$prefix .= $match if defined $match;
	$prefix .= $`;
	$match = $&;
	$s = $';
      }
      if (defined $match) {
	$s = $match . $s;
      }
    }

    my $r = {};
    my $compl = undef;
    my $ex = undef;
    foreach my $k (keys %{$charmap}) {
      if (uc(substr($k, 0, length($s))) eq uc($s)) {
	$r->{$k} = $charmap->{$k};

	if (uc($k) eq uc($s)) {
	  if (! defined $ex || $k eq $s) {
	    $ex = $k;
	  }
	  next;
	}
	if (! defined $compl) {
	  $compl = $s . substr($k, length($s));
	} else {
	  next if length($compl) <= length($s);
	  $compl = substr($compl, 0, length($k))
	    if length($k) < length($compl);
	  for (my $l = length($s) + 1;
	       length($compl) >= $l && length($k) >= $l; $l++) {
	    if (uc(substr($k, 0, $l)) ne uc(substr($compl, 0, $l))) {
	      $compl = substr($compl, 0, $l - 1);
	      last;
	    }
	  }
	}
      }
    }
    $compl = $ex if defined $ex && ! defined $compl;
    %{$r} = %{$charmap} if ! %{$r};

    return {prefix => $prefix, word => $s, completion => $compl, map => $r};
  }

 ClassInit:
  {
    use Unicode::Japanese;
    my $fn;

    $TRANSLIT_TABLE{"ascii-ascii"} = "identical";
    $TRANSLIT_TABLE{"ascii-alpha"} = "identical";
    $TRANSLIT_TABLE{"alpha-ascii"} = "identical";
    $TRANSLIT_TABLE{"alpha-alpha"} = "identical";

    sub fw2hw {
      return Unicode::Japanese->new($_[0])->z2h->getu;
    }

    sub hw2fw {
      return Unicode::Japanese->new($_[0])->h2z->getu;
    }
    sub hwkata2kata {
      my ($s) = @_;
      $s =~ s([\x{FF61}-\x{FF9F}]+){
	Unicode::Japanese->new($&)->h2z->getu;
      }gsex;
      return $s;
    }
    sub kata2hwkata {
      my ($s) = @_;
      $s =~ s([\x{3001}-\x{301C}\x{3099}-\x{309e}\x{30a1}-\x{30fe}]+){
	Unicode::Japanese->new($&)->z2h->getu;
      }gsex;
      return $s;
    }
    sub hira2kata {
      return Unicode::Japanese->new($_[0])->hira2kata->getu;
    }

    sub kata2hira {
      return Unicode::Japanese->new($_[0])->kata2hira->getu;
    }

    $TRANSLIT_TABLE{"hw-fw"} = Naggy::Translit::Function->new(\&hw2fw);
    $TRANSLIT_TABLE{"hwkata-kata"} = Naggy::Translit::Function->new(\&hwkata2kata);
    $TRANSLIT_TABLE{"fw-hw"} = Naggy::Translit::Function->new(\&fw2hw);
    $TRANSLIT_TABLE{"kata-hwkata"} = Naggy::Translit::Function->new(\&kata2hwkata);
    $TRANSLIT_TABLE{"hira-kata"} = Naggy::Translit::Function->new(\&hira2kata);
    $TRANSLIT_TABLE{"kata-hira"} = Naggy::Translit::Function->new(\&kata2hira);
    $TRANSLIT_TABLE{"hira-kata"} = Naggy::Translit::Function->new(\&hira2kata);

    $TRANSLIT_TABLE{"punctuation-fwperiod"} = Naggy::Translit::load_from_string(<<'EOT', case_sensitive => 1);
.	．
,	，
EOT

    $TRANSLIT_TABLE{"punctuation-japanese"} = Naggy::Translit::load_from_string(<<'EOT', case_sensitive => 1);
.	。
,	、
EOT


    $fn = locate_file_resource("alpha-hwkana.trl");
    my $alpha_hwkata_translit = Naggy::Translit::load_file($fn);
    die "$fn: $!" if ! defined $alpha_hwkata_translit;
    $alpha_hwkata_translit->{completion_beginning} = qr/k|s/i;
    $TRANSLIT_TABLE{"alpha-hwkata-raw"} = $alpha_hwkata_translit;

    $TRANSLIT_TABLE{"alpha-kata"} = ["alpha-hwkata", "hwkata-kata"];
    $TRANSLIT_TABLE{"alpha-hira"} = ["alpha-hwkata", "hwkata-kata",
				     "kata-hira"];
    my $hwkata_alpha_translit = $alpha_hwkata_translit->inverted_copy
      (
       pref => sub {
	 my ($a, $b) = @_;
	 if ($a =~ /^XTU|[SC]HI|FU|[X]Y[AUO]|(?:[KX]?|[SC]H|TS|KY|J)[AIUEO]$/i) {
	   return 1;
	 }
	 if ($a =~ /^N\||[ST]I|HU|[L]Y[AUO]|(?:[CL\']|[ST]Y|TX|CY|ZY)[AIUEO]$/i) {
	   return -1;
	 }
	 if ($b =~ /^XTU/) {
	   return -1;
	 }
	 my $r = length($a) <=> length($b);
#	 print "$a $b $r\n";
	 return $r;
       },
      );
    foreach my $from (keys %{$hwkata_alpha_translit->{table}}) {
      my $to = $hwkata_alpha_translit->{table}->{$from};
      if ($from =~ /^[\x{FF76}-\x{FF9C}]/) {
	$to = substr($to, 0, 1) . $to;
	$from = "ｯ" . $from;
	$hwkata_alpha_translit->put($from, $to);
      }
    }
    $hwkata_alpha_translit->put("｡", ".");
    $hwkata_alpha_translit->put("､", ",");

    $TRANSLIT_TABLE{"hwkata-alpha"} = $hwkata_alpha_translit;
    $TRANSLIT_TABLE{"kata-alpha"} = ["kata-hwkata", "hwkata-alpha"];
    $TRANSLIT_TABLE{"kana-alpha"} = ["hira-kata", "kata-hwkata", "hwkata-alpha"];
    $TRANSLIT_TABLE{"alpha-j0ja.hw"} = "alpha-hwkata";
    $TRANSLIT_TABLE{"j0ja.hw-alpha"} = "kana-alpha";

    $fn = locate_file_resource("simple_hebrew.trl");
    my $hebrew_translit = Naggy::Translit::load_file($fn);
    $TRANSLIT_TABLE{"alpha-simple_hebrew"} = $hebrew_translit;
    sub simplify_hebrew {
      my ($s) = @_;
      $s =~ s/[\x{591}-\x{5BB}\x{5BD}-\x{5C2}\x{5C4}-\x{5C7}]//g;
      $s =~ s/\x{5F0}/\x{5D5}\x{5D5}/g;
      $s =~ s/\x{5F1}/\x{5D5}\x{5D9}/g;
      $s =~ s/\x{5F2}/\x{5D9}\x{5D9}/g;
      return $s;
    }
    $TRANSLIT_TABLE{"hebrew-simplest_hebrew"}
      = Naggy::Translit::Function->new(\&simplify_hebrew);
    $TRANSLIT_TABLE{"alpha-j0heb.nes"}
      = ["alpha-simple_hebrew", "hebrew-simplest_hebrew"];
    my $hebrew_inv = $hebrew_translit->inverted_copy
      (
       pref => sub {
	 my ($a, $b) = @_;
#	 if ($a =~ /^$/i) {
#	   return 1;
#	 }
	 my $c = $a . $b;
	 if ($c =~ /^qk|vw|\`A$/) {
	   return 1;
	 }
	 if ($c =~ /^kq|wv|A\`$/) {
	   return -1;
	 }
	 if ($a =~ /\.$/i) {
	   return -1;
	 }
	 if ($b =~ /\.$/i) {
	   return 1;
	 }
	 if ($a =~ /^\{/ && $b =~ /^\{/) {
	   return length($b) <=> length($a);
	 }
	 if ($a =~ /^\{/) {
	   return -1;
	 }
	 if ($b =~ /^\{/) {
	   return 1;
	 }
	 my $r = length($a) <=> length($b);
#	 print "$a $b $r\n";
	 return $r;
       },
      );
    $TRANSLIT_TABLE{"simple_hebrew-alpha"} = $hebrew_inv;
    $TRANSLIT_TABLE{"j0heb.nes-alpha"} = "simple_hebrew-alpha";
  }
}


{
  package Naggy::Backend::CommandInterface;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  __PACKAGE__->extend_template
    (
     ngb => undef,
    );

#  __PACKAGE__->extend_cvar
#    (
#     ABBREV_echo => "echo",
#    );

#  sub echo {
#    my $self = shift;
#    $self->{ngb}->rprint(join(" ", @_));
#  }

  sub process_command {
    my $self = shift;
    my ($cmd, @args) = @_;
    $cmd =~ s/\-/_/g;
    $cmd = $self->{cvar}->{"ABBREV_" . $cmd}
      if exists $self->{cvar}->{"ABBREV_" . $cmd};

    if (defined $cmd && (ref $cmd) eq 'CODE') {
      &{$cmd}($self, @args);
      return;
    } elsif (defined $cmd && defined &{(ref $self) . "::" . $cmd}) {
      no strict;
      &{(ref $self) . "::" . $cmd}($self, @args);
      return;
    }

    $self->{ngb}->rerror("unknown command: $cmd.");
  }

  sub new {
    my $class = shift;
    my ($ngb, @rest) = @_;
    my $obj =  $class->SUPER::new(@rest);
    $obj->{ngb} = $ngb;
    return $obj;
  }
}


{
  package NES::ChrPage;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use GD;
  use Carp;

  #@NES_PAL from yychr.pal of YY-CHR.NET.
  our @NES_PAL = map {hex($_)} qw(
6b 6b 6b 00 10 84 08 00 8c 42 00 7b 63 00 5a 6b
00 10 60 00 00 4f 35 00 31 4e 18 00 5a 21 21 5a
10 08 52 42 00 39 73 00 00 00 00 00 00 00 00 00
a5 a5 a5 00 42 c6 42 29 ce 6b 00 bd 94 29 94 9c
10 42 9c 39 00 84 5e 21 5f 7b 21 2d 8c 29 18 8e
10 2e 86 63 29 73 9c 00 00 00 00 00 00 00 00 00
ef ef ef 5a 8c ff 7b 6b ff a5 5a ff d6 4a ff e7
63 9c de 7b 52 ce 9c 29 ad b5 31 7b ce 31 5a ce
52 4a c6 94 4a b5 ce 52 52 52 00 00 00 00 00 00
ef ef ef ad c6 ff bd bd ff ce b5 ff e7 b5 ff f9
bb df f7 c6 b5 de c6 9c d6 d6 94 c6 e7 9c b5 e7
ad ad e7 c6 ad de e7 ad ad ad 00 00 00 00 00 00
);

  __PACKAGE__->extend_template
    (
     chrrom => undef,
     palettes => [map {(0x00 + $_, 0x10 + $_, 0x20 + $_, 0x30 + $_)}
		  (0x0 .. 0xd)],
     pages => [],
     color => undef,
     gd => undef,
    );

  sub load_rom {
    my $self = shift;
    my ($rom) = @_;
    if ($rom !~ /^NES\x1a/) {
      my $basename = $rom;
      $basename = substr($basename, 0, 3)
	. "..." . substr($basename, -32) if length($basename) > 64;
      open(my $fh, "<", encode_fn($rom)) or croak "$basename: $!";
      binmode($fh, ":raw");
      $rom = join("", <$fh>);
      close($fh);
    }
    my $header = substr($rom, 0, 0x10);
    my ($magic, $prg_page, $chr_page, $ines_flags, $sys_flags,
	$ram_page, $tv_flags, $hd_flags) = unpack("a4CCCCCCC", $header);
    if ($magic ne "NES\x1a") {
      croak "Rom Parse Error.";
    }
    my ($mapper, $four_screen, $trainer, $sram, $vertical_mirror)
      = unpack_bits("B4BBBB", $ines_flags);
    # or # =  map {unpack("C", pack("b" . length($_), scalar reverse($_)))} unpack("a4aaaa", unpack("B8", pack("C", 0x43)))); # Crazy!
    $prg_page = $prg_page || 1;
    $chr_page = $chr_page || 1;
    $self->{chrrom} = substr($rom,
			     0x10 + $trainer * 512 + $prg_page * 0x4000,
			     $chr_page * 0x2000);
  }

  sub new {
    my $class = shift;
    my $obj =  $class->SUPER::new(@_);
    my ($rom, %opt) = @_;
    $obj->{palettes} = $opt{palettes} if exists $opt{palettes};
    $obj->load_rom($rom);
    return $obj;
  }

  sub nes_true_color {
    my $self = (ref $_[0])? shift : undef;
    my ($c) = @_;
    if ($c =~ /^[01-9]+/) {
      return  @NES_PAL[$c * 3, $c * 3 + 1, $c * 3 + 2];
    } elsif ($c =~ /^#([01-9A-Fa-f]{6})/) {
      return (hex(substr($1, 0, 2)),
	      hex(substr($1, 2, 2)),
	      hex(substr($1, 4, 2)));
    } else {
      if (exists $Resource::COLOR_NAME_LC{lc($c)}) {
	return @{$Resource::COLOR_NAME_LC{lc($c)}};
      } else {
	return ();
      }
    }
  }

  sub palette_colors {
    my $self = shift;
    my ($n) = @_;
    my $pal;
    $pal = $self->{palettes};
    my $i = $n * 4;
    return () if $i + 4 > @$pal;
    return @{$pal}[$i, $i + 1, $i + 2, $i + 3];
  }

  sub gd_page {
    my $self = shift;
    my ($num, @c) = @_;
    my $pages;
    my $rom;
    my $pal;
    $pages = $self->{pages};
    $rom = $self->{chrrom};
    $pal = $self->{palettes};
    return undef if length($rom) < ($num + 1) * 4096;
    if (defined $pages->[$num]) {
      $self->{gd} = $pages->[$num];
      $self->change_color(@c) if @c;
      return $self->{gd};
    }

    if (@c == 0) {
      if (defined $self->{color}) {
	@c = @{$self->{color}};
      } else {
	@c = $self->palette_colors(0);
      }
    }
    if (@c == 1) {
      my $i = $c[0] * 4;
      return undef if $i + 4 > @$pal;
      @c = @{$pal}[$i, $i + 1, $i + 2, $i + 3];
    }

    my $gd = GD::Image->new(128, 128, 0);
    for (my $i = 0; $i < @c; $i++) {
      my ($r, $g, $b) = nes_true_color($c[$i]);
      carp "$c[$i]: Illegal color name." if ! defined $r;
      my $id = $gd->colorAllocate($r, $g, $b);
    }
    $self->{color} = \@c;

    #$gd->transparent(0);
    for (my $i = 0; $i < 256; $i++) {
      my $addr = $num * 4096 + $i * 16;
      my $bx = ($i % 16) * 8;
      my $by = 8 * int($i / 16);
      for (my $y = 0; $y < 8; $y++) {
	my $l = ord(substr($rom, $addr + $y, 1));
	my $h = ord(substr($rom, $addr + $y + 8, 1));
	my $b = 0x80;
	for (my $x = 0; $x < 8; $x++, $b = $b >> 1) {
	  my $c = (!!($h & $b)) * 2 + (!!($l & $b));
	  $gd->setPixel($bx + $x, $by + $y, $c);
	}
      }
    }
    $pages->[$num] = $gd;
    $self->{gd} = $gd;

    return $gd;
  }

  sub get_gd {
    my $self = shift;
    my $gd = $self->{gd}->clone();
    $gd->transparent(0);
    return $gd;
  }

  sub change_color {
    my $self = ((ref $_[0])->isa(__PACKAGE__))? shift : undef;
    my $map = (defined $self)? $self->{gd} : shift;
    my (@c) = @_;
    carp "You can't change_color after setting GD::transparent()."
      if $map->transparent() != -1;
    if (@c == 1) {
      if (! defined $self) {
	$self = JRF::FlagSemaphore->get_cvar()->{CHR_PAGE};
      }
      @c = $self->palette_colors($c[0]);
    }
    for (my $i = 0; $i < @c; $i++) {
      $map->colorDeallocate($i);
      $map->colorAllocate(nes_true_color($c[$i]));
    }
    $self->{color} = \@c if defined $self;
    return $map;
  }
}


{
  package NES::RomTool;
  use base qw(JRF::Dummy);
  provide_class __PACKAGE__;

  use POSIX qw(ceil);

  __PACKAGE__->extend_template
    (
     rom => undef,
     label => {},
    );

  sub read {
    my $self = shift;
    my ($addr, $size) = @_;
    $addr = $self->{label}->{$addr}if exists $self->{label}->{$addr};
    return substr($self->{rom}, $addr, $size);
  }

  sub replace {
    my $self = shift;
    my ($addr, $size, $replace) = @_;
    $addr = $self->{label}->{$addr}if exists $self->{label}->{$addr};
    my $prev = substr($self->{rom}, $addr, $size);
    substr($self->{rom}, $addr, $size) = $replace;
  }

  sub replace_chr_page {
    my $self = shift;
    my $rom = $self->{rom};
    my ($page, $replace) = @_;
    if (ref $replace) {
      # Future Work!
    }
    $replace .= "\x00" x (0x1000 - length($replace));
    $replace = substr($replace, 0, 0x1000);
    my $header = substr($rom, 0, 0x10);
    my ($magic, $prg_page, $chr_page, $ines_flags, $sys_flags,
	$ram_page, $tv_flags, $hd_flags) = unpack("a4CCCCCCC", $header);
      my ($mapper, $four_screen, $trainer, $sram, $vertical_mirror)
	= unpack_bits("B4BBBB", $ines_flags);
    if ($page >= $chr_page * 2) {
      my $new_size = ceil($page / 2);
      my $pre = substr($rom, 0x10, $trainer * 512 + $prg_page * 0x4000);
      my $chrrom = substr($rom,
			  0x10 + $trainer * 512 + $prg_page * 0x4000,
			  $chr_page * 0x2000);
      my $rest = substr($rom, 0x10 + $trainer * 512 + $prg_page * 0x4000
			+ $chr_page * 0x2000);
      my $newchrrom = "\x00" x ($new_size * 0x2000);
      substr($newchrrom, 0, $chr_page * 0x2000) = $chrrom;
      substr($newchrrom, $page * 0x1000, 0x1000) = $replace;
      $chr_page = $new_size;
      $header = pack("a4CCCCCCC", 
		     $magic, $prg_page, $chr_page, $ines_flags, $sys_flags,
		     $ram_page, $tv_flags, $hd_flags);
       $rom = $header . $pre . $newchrrom . $rest;
    } else {
      substr($rom, 0x10 + $trainer * 512 + $prg_page * 0x4000 
	     + $page * 0x1000, 0x1000) = $replace;
    }
    $self->{rom} = $rom;
  }

  sub new {
    my $class = shift;
    my $obj =  $class->SUPER::new(@_);
    my ($rom) = @_;
    $obj->{rom} = $rom;
    return $obj;
  }
}


{
  package JRF::FlagSemaphore;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use Encode qw();

  our $ASM = locate_file_resource($Resource::SEMAPHORE_ASM);
  our $ROM = locate_file_resource($Resource::SEMAPHORE_NES_ROM);
  our $CHR_NULL = "\x00";
  our $CHR_NULL2 = "\x03";
  our $CHR_CANCEL = "\x08";
  our $SCHR_DOT = "\x{FF65}";
  our $SCHR_DASH = "\x{FF0D}";
  our $SCHR_DELIM = "\x{FF03}";
  our $SCHR_RAW = "\x{FF04}";

  __PACKAGE__->extend_template
    (
     translit => undef,
     rmode => "",
     morse_code => "",
     smode => "letter",
     bits_point => [0, 0],
     bits_mark => [0, 0],
     bits => [(0) x 256],

     filename => undef,
     opt => {},
    );

  __PACKAGE__->extend_cvar
    (
     ASM_TEXT => new_scalar(),
     CHECK_ID => new_scalar(),
     ROM_HEADER => new_scalar(),
     FLAG_CODE => [],
     FLAG_INV => {},
     MORSE_LEN_TABLE => [],
#     LOGO_CHR => {},
     SAILOR_CHR => [],
     FLAG_CHR => [],
     FLAG_LOC => [],
     CHR_PAGE => undef,
    );

  sub morse_code_to_ascii {
    my $self = ((ref $_[0])->isa(__PACKAGE__))? shift : undef;
    my $ref = shift;
    my %opt
      = (
	 error_char => "\x{FFFD}",
	 cancel_char => "\x{FFFD}",
	 cancel_mode_char => "",
	 mark_cancel => 0,
	 @_);
    my $s = (ref $ref)? $$ref : $ref;
    my $len_table = __PACKAGE__->get_cvar()->{MORSE_LEN_TABLE};
    $s =~ s/\s+//;
    $$ref = " " if ref $ref;
    my $err = $opt{error_char};
    my $cancel = $opt{cancel_char};
    my $cancel_mode = $opt{cancel_mode_char};

    my $mode = "";
    if ($s =~ s/^[\&_]//) {
      $mode = $&;
    }
    my $len = length($s);
    if ($len == 0) {
      return $mode;
    }
    $s =~ tr/\.\-/01/;
    if ($len >= 9) {
      if ($s =~ /^0+$/) {
	my $r = $cancel;
	if ((ref $ref) && $$ref ne "") {
	  $r = $cancel_mode;
	}
	$$ref = "" if ref $ref;
	return $r;
      }
      return $mode . $err;
    }
    my $c = unpack("C", (pack("b" . $len, scalar reverse($s))));
    if ($mode eq "&" && $len <= 8) {
      return chr($c);
    }
    if ($len == 8) {
      if ($c == 0) {
	my $r = $cancel;
	if ((ref $ref) && $$ref ne "") {
	  $r = $cancel_mode;
	}
	$$ref = "" if ref $ref;
	return $r;
      }
      return $mode . $err;
    }

    if ($len == 7) {
      if ($c == 0b0001001) {
	return "\$";
      }
      return $mode . $err;
    }

    $c = $len_table->[(1 << $len) - 1 + $c];
    if ($c == 0) {
      return $mode . $err;
    }
    $c = chr($c);
    if ($mode eq "_") {
      if ($c =~ tr/_\&\x27\x22\-\//_\&\`\^\~\x5c/) {
	return $c;
      }
      return uc($c);
    }
    if ($c eq "&") {
      if (ref $ref) {
	$$ref = "&";
	return "";
      } else {
	return "&";
      }
    }
    if ($c eq "_") {
      if (ref $ref) {
	$$ref = "_";
	return "";
      } else {
	return "_";
      }
    }
    return $c;
  }

  sub flag_code_to_string {
    my $self = (ref $_[0])? shift : undef;
    my $s = shift;
    my %opt
      = (
	 error_char => "\x{FFFD}",
	 cancel_char => "\x{FFFD}",
	 cancel_mode_char => "",
	 mark_cancel => "",
	 @_);
    my $r = "";
    my $morse_code = "";
    my $mode = "";
    my $flag_code;
    if (defined $self) {
      $flag_code = $self->{cvar}->{FLAG_CODE};
      $mode = $self->{rmode};
      $morse_code = $self->{morse_code};
    } else {
      $flag_code = __PACKAGE__->get_cvar()->{FLAG_CODE};
    }
    $mode = $opt{mode} if exists $opt{mode};
    my $err = encode_u($opt{error_char});
    my $cancel = encode_u($opt{cancel_char});
    my $cancel_mode = encode_u($opt{cancel_mode_char});

    while ($s ne "") {
      my $c = ord(substr($s, 0, 1));
      $s = substr($s, 1);
      my $x = $flag_code->[($c >> 4) * 8 + ($c & 7)];
      if (defined $x) {
	$x = lc(chr($x));
      } else {
	$x = $err;
      }

      if (defined $self && ($c == 0x77 || $c == 0x70 || $c == 0x07)) {
	if ($mode =~ /^\&/) {
	  if ($c == 0x07) {
	    $self->{bits_point}->[0] = $self->{bits_point}->[0] | 8;
	  } elsif ($c == 0x70) {
	    $self->{bits_point}->[0] = $self->{bits_point}->[0] & 0xF7;
	  }
	} elsif ($self->{bits_point}->[0] & 8) {
	  if ($c == 0x77) {
	    my $mask = 1 << ($self->{bits_point}->[0] & 7);
	    $self->{bits}->[$self->{bits_point}->[1]]
	      = $self->{bits}->[$self->{bits_point}->[1]] ^ $mask;
	  } elsif ($c == 0x07) {
	    $self->{bits_point}->[0]
	      = (($self->{bits_point}->[0] + 1) & 7) | 8;
	    if (($self->{bits_point}->[0] & 7) == 0) {
	      $self->{bits_point}->[1]
		= ($self->{bits_point}->[1] + 1) & 0xff;
	    }
	  } elsif ($c == 0x70) {
	    $self->{bits_point}->[0]
	      = (($self->{bits_point}->[0] - 1) & 7) | 8;
	    if (($self->{bits_point}->[0] & 7) == 0) {
	      $self->{bits_point}->[1]
		= ($self->{bits_point}->[1] - 1) & 0xff;
	    }
	  }
	} else {
	  if ($c == 0x77) {
	    @{$self->{bits_mark}} = @{$self->{bits_point}};
	  } else {
	    my $mask = 1 << ($self->{bits_point}->[0] & 7);
	    if ($c == 0x07) {
	      $self->{bits}->[$self->{bits_point}->[1]]
		= $self->{bits}->[$self->{bits_point}->[1]] & (0xff ^ $mask);
	    } elsif ($c == 0x70) {
	      $self->{bits}->[$self->{bits_point}->[1]]
		= $self->{bits}->[$self->{bits_point}->[1]] | $mask;
	    }
	    $self->{bits_point}->[0]
	      = ($self->{bits_point}->[0] + 1) & 7;
	    if (($self->{bits_point}->[0] & 7) == 0) {
	      $self->{bits_point}->[1]
		= ($self->{bits_point}->[1] + 1) & 0xff;
	    }
	  }
	}
      }

      if ($mode eq ".") {
	if ($x eq ".") {
	  $morse_code .= ".";
	  next;
	} elsif ($x eq "_" || $x eq "f") {
	  $morse_code .= "-";
	  $mode = "";
	  next;
	} elsif ($x eq "\x00") {
	  $morse_code .= ".";
	  $mode = "";
	  next;
	} else {
	  $morse_code .= ".";
#	  print "MORSE_OUT \"$morse_code\"\n";
	  $r .= encode_u(morse_code_to_ascii(\$morse_code, %opt));
	  $mode = "";
	}
      }
 #     print "\"$r\" \"$mode\"\n";
      if ($x eq "." && $mode ne "_") {
#	print "MORSE_IN\n";
	if ($mode ne "" && $mode ne "\$") {
	  $r .= $err;
	}
	if ($morse_code =~ /^[\$_ ] /) {
	  $r .= " ";
	}
	$morse_code =~ s/\s+//;
	$mode = ".";
	next;
      }
      if ($x ne "." && $morse_code !~ /^\s*$/) {
#	print "MORSE_OUT \"$morse_code\" \"$r\"\n";
	$r .= encode_u(morse_code_to_ascii(\$morse_code, %opt));
	$mode = "";
      }
      if ($x eq "_" && $mode ne "_") {
	if ($mode ne "" && $mode ne "\$") {
	  $r .= $err;
	  $mode = "";
	}
	$mode = $mode . "_";
	next;
      }
      if ($x eq $CHR_NULL) {
	if (length($morse_code) > 0 && length($morse_code) < 3)  {
	  $morse_code .= " ";
	}
	next;
      }
      if ($x eq $CHR_NULL2) {
	$morse_code =~ s/ +//;
	next;
      }
      if ($x eq $CHR_CANCEL) {
	if ($morse_code eq "" && $mode eq "") {
	  $r .= $cancel;
	} else {
	  $r .= $cancel_mode;
	}
	$morse_code = "";
	$mode = "";
	next;
      }
      if ($x eq "\$") {
	$morse_code = "";
	if ($mode ne "" && $mode ne "\$") {
	  $r .= $err;
	}
	$mode = "\$";
	next;
      }
      if ($x eq "\&") {
	$morse_code = "";
	if ($mode ne "" && $mode ne "\$") {
	  $r .= $err;
	}
	$mode = "\&";
	next;
      }
      if ($x eq 'j' && $mode =~ /^\$/) {
	if (length($') > 0) {
	  $r .= $err;
	}
	$mode = "";
	next;
      }

      if ($mode eq "\$_") {
	if ($x eq 'k') {
	  $r .= "0";
	  $mode = "\$";
	  next;
	} elsif (ord($x) >= ord("a") && ord($x) <= ord("i")) {
	  $r .= chr(ord($x) - ord("a") + ord("1"));
	  $mode = "\$";
	  next;
	} else {
	  $mode = "\$";
	  $r .= $err;
	  next;
	}
      }
      if ($mode eq "\&_") {
	if ($x eq 'k') {
	  $r .= "0";
	  $mode = "";
	  next;
	} elsif (ord($x) >= ord("a") && ord($x) <= ord("i")) {
	  $r .= chr(ord($x) - ord("a") + ord("1"));
	  $mode = "";
	  next;
	} else {
	  $mode = "";
	  $r .= $err;
	  next;
	}
      }
      if ($mode =~ /^[\$\&]([01-9]*)/) {
	my $oct = $1;
	if ($x eq '_' && $oct eq "") {
	  $mode .= "_";
	  next;
	} elsif ($x eq 'k') {
	  $oct .= "0";
	} elsif (ord($x) >= ord("a") && ord($x) <= ord("i")) {
	  $oct .= chr(ord($x) - ord("a") + ord("1"));
	} else {
	  $mode = ($mode =~ /^\$/)? "\$" : "";
	  $r .= $err;
	  next;
	}
	if (length($oct) == 2 && substr($oct, 0, 1) =~ /^[4-9]$/) {
	  my $l = substr($oct, 1, 1);
	  my $m = substr($oct, 0, 1);
	  my $h = 0;
	  if ($m == 8) {
	    $h = 1;
	    $m = ($l == 0)? 0 : 3;
	  } elsif ($m == 9) {
	    $h = 1;
	    $m = ($l == 0)? 4 : 7;
	  }
	  $r .= chr(($h << 6) | ($m << 3) | $l);
	  $mode = ($mode =~ /^\$/)? "\$" : "";
	  next;
	} elsif (length($oct) == 3) {
	  my $l = substr($oct, 2, 1);
	  my $m = substr($oct, 1, 1);
	  my $h = substr($oct, 0, 1);
	  $r .= chr(($h << 6) | ($m << 3) | $l);
	  $mode = ($mode =~ /^\$/)? "\$" : "";
	  next;
	} else {
	  $mode = substr($mode, 0, 1) . $oct;
	  next;
	}
      }
      if ($mode eq '_') {
	$mode = "";
	if ($x eq ".") {
	  $r .= ".";
	  next;
	} elsif ($x eq " ") {
	  $r .= ",";
	  next;
	}
	$x = uc($x);
      }
      $r .= $x;
    }
    if ((! defined $self) || (exists $opt{flush} && $opt{flush})) {
      if ($mode eq ".") {
	$morse_code .= ".";
	$mode = "";
	$r .= encode_u(morse_code_to_ascii(\$morse_code, %opt));
      }
      $morse_code =~ s/ +$//;
      $r .= $morse_code;
      $morse_code = "";
      if ($mode ne "" && $mode ne "\$") {
	$r .= $err;
      }
      $mode = "";
    }
    if (defined $self) {
      $self->{rmode} = $mode;
      $self->{morse_code} = $morse_code;
    }

    $r = decode_u($r);
    return $r;
  }

  sub string_to_flag_code {
    my $self = (ref $_[0])? shift : undef;
    my ($s, %opt) = @_;
    my $r;
    $s = encode_u($s);
    if (defined $self) {
      $r = $self->ascii_to_flag_code($s, %opt);
    } else {
      $r = ascii_to_flag_code($s, %opt);
    }
    return $r;
  }

#  sub _raw_ascii_to_flag_code {
#    my ($tbl, $c) = @_;
#    $c = ord(lc($c));
#    if (exists $tbl->{$c}) {
#      return chr($c);
#    }
#    return undef;
#  }

  sub _concat_flag_code {
    my ($a, $b) = @_;
    return $a if $b eq "";
    return $b if $a eq "";
    my $la = substr($a, -1);
    my $bb = substr($b, 0, 1);
    if ($la eq $bb && $la ne "\x00") {
      return $a . "\x00" . $b;
    }
    return $a . $b;
  }

  sub ascii_to_flag_code {
    my $self = (ref $_[0])? shift : undef;
    my ($s, %opt) = @_;
    my $r = "";
    my $mode = "letter";
    my $flag_inv;
    if (defined $self) {
      $mode = $self->{smode};
      $flag_inv = $self->{cvar}->{FLAG_INV};
    } else {
      $flag_inv = __PACKAGE__->get_cvar()->{FLAG_INV};
    }
    $mode = $opt{mode} if exists $opt{mode};

    while ($s ne "") {
      my $c = substr($s, 0, 1);
      my $x = ord($c) & 0xFF;
      $s = substr($s, 1);
      my $d;
      my $next;

      if ($c eq $SCHR_DOT) {
	$d = "\x04\x00";
	$next = "morse";
      } elsif ($c eq $SCHR_DASH) {
	$d = "\x04\x06";
	$next = "morse";
      } elsif ($c eq $SCHR_DELIM) {
	$d = "\x07";
	$next = $mode;
      } elsif ($c eq $SCHR_RAW) {
	$d = "";
	$next = "raw";
      } elsif ($x >= ord("a") && $x <= ord("z")) {
	$d = chr($flag_inv->{$x});
	$next = "letter";
      } elsif ($x >= ord("A") && $x <= ord("Z")) {
	$d = _concat_flag_code(chr($flag_inv->{ord("_")}),
			       chr($flag_inv->{$x + ord("a") - ord("A")}));
	$next = "letter";
      } elsif ($x >= ord("0") && $x <= ord("9")) {
	if ($mode eq "raw") {
	  $d = chr($flag_inv->{$x});
	  $next = "raw";
	} else {
	  $d = _concat_flag_code(chr($flag_inv->{ord("_")}),
				 chr($flag_inv->{$x}));
	  $next = "number";
	}
      } elsif ($x == ord(" ")) {
	$d = chr($flag_inv->{ord(" ")});
	$next = $mode;
      } elsif ($x == ord(".")) {
	$d = _concat_flag_code(chr($flag_inv->{ord("_")}),
			       chr($flag_inv->{$x}));
	$next = $mode;
      } elsif ($x == ord(",")) {
	$d = _concat_flag_code(chr($flag_inv->{ord("_")}),
			       chr($flag_inv->{ord(" ")}));
	$next = $mode;
      } else {
	my $l = $x & 0x7;
	my $m = ($x >> 3) & 0x7;
	my $h = ($x >> 6) & 0x7;
	$d = chr($flag_inv->{$l + ord("0")});
	if ($h == 0 && $m >= 4) {
	  $d = _concat_flag_code(chr($flag_inv->{$m + ord("0")}), $d);
	} elsif (($h == 1 && $m == 0 && $l == 0)
		 || ($h == 1 && $m == 3 && $l != 0)) {
	  $d = _concat_flag_code(chr($flag_inv->{ord("8")}), $d);
	} elsif (($h == 1 && $m == 4 && $l == 0)
		 || ($h == 1 && $m == 7 && $l != 0)) {
	  $d = _concat_flag_code(chr($flag_inv->{ord("9")}), $d);
	} else {
	  $d = _concat_flag_code(chr($flag_inv->{$m + ord("0")}), $d);
	  $d = _concat_flag_code(chr($flag_inv->{$h + ord("0")}), $d);
	}
	if ($mode eq "letter"
	    && $s ne "" && substr($s, 0, 1) =~ /^[A-Za-z]/) {
	  $d = _concat_flag_code(chr($flag_inv->{ord("\&")}), $d);
	  $next = "letter";
	} else {
	  $next = "number";
	}
      }

      if ($mode ne $next) {
	if ($next eq "letter" && ($mode eq "raw" || $mode eq "number")) {
	  $d = _concat_flag_code(chr($flag_inv->{ord("j")}), $d);
	} elsif (($next eq "raw" || $next eq "number") &&
		 ! ($mode eq "raw" || $mode eq "number")) {
	  $d = _concat_flag_code(chr($flag_inv->{ord("\$")}), $d);
	}
	$mode = $next;
      }
      $r = _concat_flag_code($r, $d);
    }
    if (defined $self) {
      $self->{smode} = $mode;
    }
    return $r;
  }

  sub new_sailor {
    my $self = shift;
    return (__PACKAGE__ . "::Sailor")->new($self, @_);
  }

  sub generate_sav {
    my $self = shift;
    my $s = shift;
    my %opt = (font8x16 => 0,
	       toprintable => 1,
	       righttoleft => 0,
	       chr_page8k => 2,
	       font => 1,
	       color => 1,
	       @_);
    $s = encode_u($s);
    if (length($s) > 255) {
      $s = substr($s, -255);
    }
    my $len = length($s);
    $s .= "\x00" x (0x100 - $len);
    my $sav = "\x00" x 0x2000;
    my $check_id = ${$self->{cvar}->{CHECK_ID}};
    substr($sav, 0, length($check_id)) = $check_id;
    substr($sav, 0x10, 1) = pack("C", $len);
    my $opt = pack_bits("B4BBBB", $opt{chr_page8k}, 0, 
			$opt{righttoleft}, $opt{toprintable}, $opt{font8x16});
    $opt = $opt{sav_switch}
      if exists $opt{sav_switch} && defined $opt{sav_switch};
    substr($sav, 0x13, 1) = pack("C", $opt);
    substr($sav, 0x14, 1) = pack("C", $opt{font}) if exists $opt{font};
    substr($sav, 0x15, 1) = pack("C", $opt{color}) if exists $opt{color};
    substr($sav, 0x20, 4) = pack("C4", @{$self->{bits_point}},
				 @{$self->{bits_mark}});
    substr($sav, 0x100, 0x100) = $s;
    substr($sav, 0x200, 0x100) = pack("C256", @{$self->{bits}});
    return $sav;
  }

  sub new {
    my $class = shift;
    my $obj =  $class->SUPER::new(@_);
    if (@_ % 2 == 1) {
      unshift(@_, "file");
    }
    %{$obj->{opt}} = (%{$obj->{opt}}, @_);
    return $obj;
  }

 ClassInit:
  {
    my $cvar = __PACKAGE__->get_cvar();
    my $asm;
    my $rom;
    my $version;
    my $check_id;
    my $tables = {};
    my $tables_num;

    my $parse_data = sub {
      my ($line) = @_;
      $line =~ s/\;.*$//s;
      if ($line !~ /^\s*\.?(?:db|dw|byte|word)\s+/is) {
	return ();
      }
      my (@l) = split(/\s*,\s*/, $');
      my @r;
      foreach my $c (@l) {
	if ($c =~ /^\s*\"(.*)\"\s*$/s) {
	  push(@r, ["number", unpack("C*", $1)]);
	} elsif ($c =~ /^\s*\'(.)\'\s*$/s) {
	  push(@r, ["number", ord($1)]);
	} elsif ($c =~ /^\s*\%([01]+)\s*$/s) {
	  push(@r, ["number", unpack("C",  pack("B8", $1))]);
	} elsif ($c =~ /^\s*\$([01-9A-Fa-f]+)\s*$/s) {
	  push(@r, ["number",hex($1)]);
	} elsif ($c =~ /^\s*([01-9]+)\s*$/s) {
	  push(@r, ["number", int($1)]);
	} elsif ($c =~ /^\s*([A-Za-z01-9_]+)\s*$/s) {
	  push(@r, ["symbol", $1]);
	} else {
	  die "$ASM: Parse Error.";
	}
      }
      @l = @r;
      @r = ();
      while (@l) {
	my $c = shift(@l);
	if ($c->[0] ne "symbol") {
	  while (@l) {
	    my $d = $l[0];
	    last if $d->[0] eq "symbol";
	    shift(@l);
	    shift(@$d);
	    shift(@$c);
	    $c = ["number", @$c, @$d];
	  }
	}
	push(@r, $c);
      }
      return @r;
    };

    {
      open(my $fh, "<", encode_fn($ASM)) or die "$ASM: $!";
      binmode($fh, ":utf8");
      $asm = join("", <$fh>);
      close($fh);
    }

    {
      open(my $fh, "<", encode_fn($ROM)) or die "$ASM: $!";
      binmode($fh);
      $rom = join("", <$fh>);
      close($fh);
    }

    ${$cvar->{ASM_TEXT}} = $asm;

    {
      my $s = $asm;
      if ($s !~ /^((?:[^\n]*\n){0,16})/s) {
	die "$ASM: Parse Error.";
      }
      $s = $1;
      if ($s !~ /(?:JRF_VERSION[^\n]*\n?[^\n]*[\"']([^\"'\s]+)[\"'])/s) {
	die "$ASM: Parse Error.$s";
      }
      $version = $1;
    }

    {
      my $s = $asm;
      my @r;
      my @n;
      if ($s !~ /\nCheckID\:(\s+\.?(?:db|byte|DB|BYTE)[^\n]+)/s) {
	die "$ASM: Parse Error.";
      }
      $s = $1 . $';
      while ($s =~ /^\s+(\.?(?:db|byte)\s+[^\n]+)/is) {
	$s = $';
	@r = &{$parse_data}($1);
	if (@r != 1 && $r[0]->[0] ne "number") {
	  die "$ASM: Parse Error.";
	}
	shift(@{$r[0]});
	push(@n, @{$r[0]});
	last if grep {$_ == 0} @n;
      }
      if (! grep {$_ == 0} @n) {
	  die "$ASM: Parse Error.";
      }
      @r = ();
      while (@n) {
	my $c = shift(@n);
	last if $c == 0;
	push(@r, $c);
      }
      if (@r == 0) {
	die "$ASM: Parse Error.";
      }
      $check_id = pack("C*", @r);
      ${$cvar->{CHECK_ID}} = $check_id;
    }

    {
      my $s = $asm;
      if (($s !~ /\nPublicSymbols\:[^\n]*\n(.*\n)PublicSymbolsEnd\:/s) 
	  || ($s !~ /\nTables\:[^\n]*\n(.*\n)TablesEnd\:/s)) {
	die "$ASM: Parse Error.";
      }
      $s = $1;
      my @r;
      while ($s =~ /^([^\n]+)\n/s) {
	$s = $';
	my $c = $1;
	push(@r, &{$parse_data}($c)) if $c !~ /^[A-Za-z\._]/;
      }
      if (@r == 0 || ! ($r[$#r]->[0] eq "number" && $r[$#r]->[1] == 0)) {
	die "$ASM: Parse Error.";
      }
      $tables_num = scalar @r;
      for (my $i = 0; $i < @r; $i++) {
	my $c = $r[$i];
	if ($c->[0] eq "symbol") {
	  my $e = $c->[1];
	  $e =~ s/^_//;
	  $tables->{$e} = {};
	  $tables->{$e}->{num} = $i;
	}
      }
      if (! exists $tables->{"CheckID"}) {
	die "$ASM: Parse Error.";
      }
    }

    {
      my $s = $rom;
      if ($s !~ /\Q$check_id\E\x00[^\x00]*\Q$version\E[^\x00]*\x00/s) {
	die "$ROM: Parse Error.";
      }
      my $id_addr = length($`);
      my $tb_addr = $id_addr + length($&);
      my @addr = unpack("S*", pack("s*", unpack("v*", substr($', 0, 2 * $tables_num))));
      if ($addr[$#addr] != 0) {
	die "$ROM: Parse Error.";
      }
      my $id_raddr = $addr[$tables->{"CheckID"}->{num}];
      foreach my $k (keys %$tables) {
	my $ra = $addr[$tables->{$k}->{num}];
	$tables->{$k}->{addr} = $addr[$tables->{$k}->{num}] - $id_raddr + $id_addr;
      }
    }

    if (! exists $tables->{"FlagCode"}) {
      die "$ROM: Parse Error.";
    }
    @{$cvar->{FLAG_CODE}} = unpack("C*", substr($rom, $tables->{"FlagCode"}->{addr}, 8 * 8));

    if (! exists $tables->{"MorseCodeLenTable"}) {
      die "$ROM: Parse Error.";
    }
    @{$cvar->{MORSE_LEN_TABLE}}
      = unpack("C*", substr($rom, $tables->{"MorseCodeLenTable"}->{addr}, (1 << (6 + 1)) - 1));

    if (! exists $tables->{"SailorChr"}) {
      die "$ROM: Parse Error.";
    }
    {
      my @r;
      my $addr = $tables->{"SailorChr"}->{addr};

      for (my $i = 0; $i < 2 * 4; $i++) {
	push(@r, [unpack("C*", substr($rom, $addr + 5 * $i, 5))]);
      }
      @{$cvar->{SAILOR_CHR}} = @r;
    }

    if (! exists $tables->{"FlagChr"}) {
      die "$ROM: Parse Error.";
    }
    {
      my @r;
      my $addr = $tables->{"FlagChr"}->{addr};

      for (my $i = 0; $i < 32; $i++) {
	push(@r, [unpack("C*", substr($rom, $addr + 5 * $i, 5))]);
      }
      @{$cvar->{FLAG_CHR}} = @r;
    }

    if (! exists $tables->{"FlagLoc"}) {
      die "$ROM: Parse Error.";
    }
    {
      my @r;
      my $addr = $tables->{"FlagLoc"}->{addr};

      for (my $i = 0; $i < 16; $i++) {
	push(@r, [unpack("c*", substr($rom, $addr + 2 * $i, 2))]);
      }
      @{$cvar->{FLAG_LOC}} = @r;
    }

    {
      my @flag_code = @{$cvar->{FLAG_CODE}};
      my %flag_inv = ();
      for (my $l = 0; $l < 8; $l++) {
	for (my $r = 0; $r < 8; $r++) {
	  my $v = $flag_code[$l * 8 + $r];
	  my $c = ($l  << 4) | $r;
	  next if $v == ord('?');
	  next if lc(chr($v)) ne chr($v);
	  if (! exists $flag_inv{$v}) {
	    $flag_inv{$v} = $c;
	  } elsif ($v == 0x03 && $c == 0x07) {
	    $flag_inv{$v} = $c;
	  }
	}
      }
      $flag_inv{ord('0')} = $flag_inv{ord('k')};
      for (my $i = ord('1'); $i <= ord('9'); $i++) {
	$flag_inv{$i} = $flag_inv{$i - ord('1') + ord('a')};
      }
      %{$cvar->{FLAG_INV}} = %flag_inv;
    }

    ${$cvar->{ROM_HEADER}}  = substr($rom, 0, 0x10);
    if (! exists $tables->{"PaletteSize"}) {
      die "$ROM: Parse Error.";
    }
    if (! exists $tables->{"Palette"}) {
      die "$ROM: Parse Error.";
    }
    {
      my $sz = unpack("C", substr($rom, $tables->{"PaletteSize"}->{addr}, 1));
      my $pal = [unpack("C*", substr($rom, $tables->{"Palette"}->{addr}, $sz))];
      $cvar->{CHR_PAGE} = NES::ChrPage->new($rom, palettes => $pal);
    }
  }
}

{
  package JRF::FlagSemaphore::Sailor;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use GD;

  __PACKAGE__->extend_template
    (
     j => undef,
     font => 1,
     bg => undef,
     spr => undef,
     spw => undef,
     sprh => undef,
     spwh => undef,
     bgcolor => undef,
     size => [48, 48],
     center => [24, 32],
    );

  sub new {
    my $class = shift;
    my ($j, %opt) = @_;
    my $obj =  $class->SUPER::new(@_);
    $obj->{j} = $j;
    my $cp = $j->{cvar}->{CHR_PAGE};
    my @c;
    if (exists $opt{color}) {
      if (ref $opt{color}) {
	@c = @{$opt{color}};
      } else {
	@c = $cp->palette_colors($opt{color});
      }
    }
    @c = $cp->palette_colors(1) if ! @c;
    $cp->gd_page(0, @c);
    $obj->{bg} = $cp->{gd}->clone();
    my @sc = $cp->palette_colors(5);
    $sc[3] = $c[3];
    $cp->change_color(@sc);
    $obj->{spw} = $cp->{gd}->clone();
    $obj->{spwh} = $obj->{spw}->clone();
    $obj->{spwh}->flipHorizontal();
    @sc = $cp->palette_colors(7);
    $sc[3] = $c[3];
    $cp->change_color(@sc);
    $obj->{spr} = $cp->{gd}->clone();
    $obj->{sprh} = $obj->{spr}->clone();
    $obj->{sprh}->flipHorizontal();

    $obj->{bg}->transparent(0);
    $obj->{spw}->transparent(0);
    $obj->{spwh}->transparent(0);
    $obj->{spr}->transparent(0);
    $obj->{sprh}->transparent(0);

    if (exists $opt{font}) {
      my $f = $opt{font};
      if ($f =~ /^[01-9]+$/) {
	$obj->{font} = $f & 7;
      } else {
	my $n = 0;
	if ($f =~ /italic/i) {
	  $n = $n | 1;
	}
	if ($f =~ /slant/i) {
	  $n = $n | 2;
	}
	if ($f =~ /bold/i) {
	  $n = $n | 4;
	}
	$obj->{font} = $n;
      }
    }

    my $bg;
    if (exists $opt{bgcolor}) {
      $bg = $opt{bgcolor};
      if (! ref $bg) {
	$bg = [$cp->nes_true_color($bg)];
      }
    }
    if (! defined $bg || @$bg < 3) {
      $bg = [$cp->nes_true_color(($cp->palette_colors(4))[0])];
    }
    $obj->{bgcolor} = $bg;

    return $obj;
  }

  sub _draw_chr {
    my $self = shift;
    my ($dest, $src, $srch, $spec, $x, $y) = @_;
    my ($col, @chr) = @$spec;
    if ($col & 0x40) {
      @chr = map {16 * int($_ / 16) + 15 - ($_ % 16)} @chr;
      $src = $srch;
    }
    my $c = $chr[0];
    $dest->copy($src, $x, $y, 8 * ($c % 16), 8 * int($c / 16), 8, 8);
    $c = $chr[1];
    $dest->copy($src, $x + 8, $y, 8 * ($c % 16), 8 * int($c / 16), 8, 8);
    $c = $chr[2];
    $dest->copy($src, $x, $y + 8, 8 * ($c % 16), 8 * int($c / 16), 8, 8);
    $c = $chr[3];
    $dest->copy($src, $x + 8, $y + 8, 8 * ($c % 16), 8 * int($c / 16), 8, 8);
  }

  sub flag {
    my $self = shift;
    my ($c) = @_;
    my $sailor_chr = $self->{j}->{cvar}->{SAILOR_CHR};
    my $flag_chr = $self->{j}->{cvar}->{FLAG_CHR};
    my $flag_loc = $self->{j}->{cvar}->{FLAG_LOC};
    my $sfont = $self->{font} >> 1;
    my $ffont = $self->{font} & 1;
    my $gd = GD::Image->new(@{$self->{size}});
    $gd->colorAllocate(@{$self->{bgcolor}});
    $gd->filledRectangle(0, 0, @{$self->{size}}, 0);
    my ($bx, $by) = @{$self->{center}};
    $bx -= 8;
    $by -= 16;
    my $lred = (ord($c) >> 4) & 0x7;
    my $rwhite = ord($c) & 0x7;
    my ($lx, $ly) = @{$flag_loc->[$lred]};
    my ($rx, $ry) = @{$flag_loc->[8 + $rwhite]};

    $self->_draw_chr($gd, $self->{bg}, $self->{bg},
		     $sailor_chr->[$sfont * 2], $bx, $by);
    $self->_draw_chr($gd, $self->{bg}, $self->{bg},
		     $sailor_chr->[$sfont * 2 + 1], $bx, $by + 16);
    if ($rwhite > $lred) {
      $self->_draw_chr($gd, $self->{spr}, $self->{sprh},
		       $flag_chr->[$ffont * 16 + $lred],
		       $bx + $lx, $by + $ly);
      $self->_draw_chr($gd, $self->{spw}, $self->{spwh},
		       $flag_chr->[$ffont * 16 + 8 + $rwhite],
		       $bx + $rx, $by + $ry);
    } else {
      $self->_draw_chr($gd, $self->{spw}, $self->{spwh},
		       $flag_chr->[$ffont * 16 + 8 + $rwhite],
		       $bx + $rx, $by + $ry);
      $self->_draw_chr($gd, $self->{spr}, $self->{sprh},
		       $flag_chr->[$ffont * 16 + $lred],
		       $bx + $lx, $by + $ly);
    }
    return $gd;
  }

  sub flag_gif {
    my $self = shift;
    my ($s, %opt) = @_;
    my $bg;
    $bg = $opt{bgcolor} if exists $opt{bgcolor};
    $bg = $opt{bg} if exists $opt{bg};
    my ($w, $h) = @{$self->{size}};
    my ($bx, $by) = (0, 0);
    if (exists $opt{center}) {
      ($bx, $by) = @{$opt{center}};
      $bx = int($bx - $w * 0.5);
      $by = int($by - $h * 0.5);
    }
    if (exists $opt{size}) {
      ($w, $h) = @{$opt{size}};
    }
    $w = $opt{width} if exists $opt{width};
    $h = $opt{height} if exists $opt{height};
    $bg = $self->{bgcolor} if ! defined $bg;
    if (defined $bg && ! ref $bg) {
      $bg = [$self->{j}->{CHR_PAGE}->nes_true_color($bg)];
    }
    if ('ARRAY' eq (ref $bg)) {
      my $gd = GD::Image->new($w, $h);
      $gd->colorAllocate(@$bg);
      $gd->filledRectangle(0, 0, $w, $h, 0);
      $bg = $gd;
    }
    my $loop = 0;
    $loop = $opt{loop} if exists $opt{loop};
    my $interval = 100;
    $interval = $opt{interval} if exists $opt{interval};
    my $gifanim;

    while ($s ne "") {
      my $c = substr($s, 0, 1);
      $s = substr($s, 1);
      my $gd = $self->flag($c);
      if (defined $bg) {
	$gd->transparent(0);
	my $gd2 = $bg->clone();
	$gd2->copy($gd, $bx, $by, 0, 0, $gd->width, $gd->height);
	$gd = $gd2;
      }
      if (! defined $gifanim) {
	return $gd->gif() if $s eq "";
	$gifanim = $gd->gifanimbegin(0, $loop);
      }
      $gifanim .= $gd->gifanimadd(1, 0, 0, $interval);
      $gifanim .= $gd->gifanimend() if $s eq "";
    }
    return $gifanim;
  }
}

{
  package GifFont;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use GD;

  __PACKAGE__->extend_template
    (
     gd => undef,
     orig_opt => {},

     bgcolor => [0x10, 0x43, 0x1f],
     font_width => 8,
     font_height => 8,
     char_width => 8,
     line_height => 16,
     margin_top => 8,
     margin_left => 8,
     margin_bottom => 8,
     margin_right => 8,
     cols => 22,
     address_map => [],
     direction => "ltor",
     rotate => 0,
     first_point => [0, 0],
    );

#  __PACKAGE__->extend_cvar
#    (
#    );

  our %OPT_ABBREV
    = (
       s => "size",
       w => "font_width",
       h => "font_heigth",
       cw => "char_width",
       bg => "bgcolor",
       l => "line_height",
       lh => "line_height",
       m => "margin",
       a => "address",
       r => "rotate",
       d => "direction",
       pal => "palette",
       f => "first_point",
      );

  sub new {
    my $class = shift;
    my $self =  $class->SUPER::new(@_);
    my (@opt) = @_;
    my @addr;
    my %opt;
    if (@opt % 2) {
      $opt{file} = shift(@opt);
    }
    while (@opt) {
      my $k = shift(@opt);
      my $v = shift(@opt);
      my $key = $k;
      $key =~ s/^\-\-?//;
      $key =~ s/\-/_/g;
      $key = $OPT_ABBREV{$key} if exists $OPT_ABBREV{$key};
      if ($key eq "address") {
	push(@addr, $v);
      } else {
	$opt{$key} = $v;
      }
    }
    $self->{orig_opt} = \%opt;

    if (exists $opt{size}) {
      my ($w, $h);
      if (ref $opt{size}) {
	($w, $h) = @{$opt{size}};
      } else {
	($w, $h) = split("x", $opt{size});
      }
      $self->{font_width} = $w;
      $self->{font_height} = $h;
    }

    if (exists $opt{first_point}) {
      my ($x, $y);
      if (ref $opt{first_point}) {
	($x, $y) = @{$opt{first_point}};
      } else {
	($x, $y) = split(/\s*,\s*/, $opt{first_point});
      }
      $self->{frist_point} = [$x, $y];
    }

    if (exists $opt{margin}) {
      my ($top, $left, $bottom, $right) = split(/\s*,\s*/, $opt{margin});
      $top = 0 if ! defined $top;
      $left = $top if ! defined $left;
      $bottom = $top if ! defined $bottom;
      $right = $left if ! defined $right;
      $self->{margin_top} = $top;
      $self->{margin_left} = $left;
      $self->{margin_bottom} = $bottom;
      $self->{margin_right} = $right;
    }

    foreach my $k qw(char_width line_height font_width 
		     font_height rotate direction gd) {
      $self->{$k} = $opt{$k} if exists $opt{$k};
    }

    $self->{direction} = lc($self->{direction});
    $self->{direction} =~ s/right/r/g;
    $self->{direction} =~ s/left/l/g;
    $self->{direction} =~ s/\-//g;

    foreach my $addr (@addr) {
      my ($range, $map) = split(/\:/, $addr);
      if (! defined $map) {
	$map = "0-FF";
      }
      my ($from, $to) = map {parse_int($_, 16)} split(/\-/, $range);
      my ($mfrom, $mto) = map {parse_int($_, 16)} split(/\-/, $map);
      my ($hdir, $vdir) = (1, 1);
      if (defined $mto && ($mfrom eq "" 
			   || ($mto ne "" && ($mto - $mfrom) < 0))) {
	$hdir = -1;
	$vdir = -1;
      }
      $to = $from if ! defined $to;
      $to = undef if $to eq "";
      push(@{$self->{address_map}}, [$from, $to, $mfrom, $hdir, $vdir]);
    }

    return $self;
  }

  sub get_true_color {
    my $self = ((ref $_[0])->isa(__PACKAGE__))? shift : undef;
    my ($c) = @_;
    return @$c if ref $c;
    if (exists $Resource::COLOR_NAME_LC{lc($c)}) {
      return @{$Resource::COLOR_NAME_LC{lc($c)}};
    } else {
      return ();
    }
  }

  sub get_cmap {
    my $self = shift;
    return $self->{gd} if defined $self->{gd};
    if (exists $self->{orig_opt}->{file}) {
      $self->{gd} = GD::Image->new($self->{orig_opt}->{file});
    }
    return $self->{gd};
  }

  sub text_gd {
    my $self = shift;
    my ($s, %opt) = @_;
    my $cmap = $self->get_cmap();
    my @line;
    my $gd;
    my $col;
    my $x;
    my $row = 0;
    my $bgcolor = [$self->get_true_color($self->{bgcolor})];
    my @amap = @{$self->{address_map}};
    while ($s ne "") {
      my $c = substr($s, 0, 1);
      $s = substr($s, 1);
      my $o = ord($c);
      if (! defined $gd) {
	my $w = $self->{margin_left} + $self->{char_width} * $self->{cols}
	  + $self->{margin_right};
	my $h = $self->{line_height};
	$gd = GD::Image->new($w, $h);
	$gd->colorAllocate(@$bgcolor);
	$gd->filledRectangle(0, 0, $w, $h, 0);
	$gd->transparent(0);
	if ($self->{direction} eq "rtol") {
	  $x = $self->{margin_left} + $self->{char_width} * ($self->{cols} - 1);
	} else {
	  $x = $self->{margin_left};
	}
	$col = 0;
	$row++;
      }
      foreach my $p (@amap) {
	my ($from, $to, $mfrom, $hdir, $vdir) = @$p;
	next if $o < $from;
	next if defined $to && $o > $to;
	my $fx = $mfrom % 16;
	my $fy = int($mfrom / 16);
	my $dx = ($o - $from) % 16;
	my $dy = int(($o - $from) / 16);
	my $cx = $fx + $hdir * $dx;
	$dy += abs(int($cx / 16));
	$cx = $cx % 16;
	my $cy = $fy + $dy * $vdir;
	$o = $cx + $cy * 16;
      }

      if ($self->{font_width} * 2 <= $self->{font_height}) {
	if ($o & 0x80) {
	  $o = ($o + ($o & 0xf0)) | 0x10;
	  if ($o & 0x80) {
	    $o = ~$o;
	  }
	}
	$o = $o & 0x7F;
      } else {
	$o = $o & 0xFF;
      }
      my ($fx, $fy) = @{$self->{first_point}};
      $fx += ($o % 16) * $self->{font_width};
      $fy += $self->{font_height} * int($o / 16);
      $gd->copyRotated($cmap, $x + ($self->{char_width} / 2),
		       $self->{line_height} / 2, $fx, $fy,
		       $self->{font_width}, $self->{font_height},
		       $self->{rotate});
      if ($self->{direction} eq "rtol") {
	$x -= $self->{char_width};
      } else {
	$x += $self->{char_width};
      }
      $col++;
      if ($col >= $self->{cols}) {
	push(@line, $gd);
	$gd = undef;
      }
    }
    push(@line, $gd) if defined $gd;
    if (1) {
      my $w = $self->{margin_left} + $self->{char_width} * $self->{cols}
	+ $self->{margin_right};
      my $h = $self->{margin_top} + ($self->{line_height} * @line)
	+ $self->{margin_top};
      my $gd = GD::Image->new($w, $h);
      $gd->colorAllocate(@$bgcolor);
      $gd->filledRectangle(0, 0, $w, $h, 0);
      my $y = $self->{margin_top};
      foreach my $l (@line) {
	$gd->copy($l, 0, $y, 0, 0, $l->width, $l->height);
	$y += $l->height;
      }
      return $gd;
    }
  }
}

{
  package GifFont::JRFSemaphoreRom;
  use base qw(GifFont);
  provide_class __PACKAGE__;

  use GD;
  use Carp;

  __PACKAGE__->extend_template
    (
     page => 0,
     palette => 0,
     chr_page => undef,
    );

  sub get_true_color {
    my $self = shift;
    my ($c) = @_;
    return @$c if ref $c;
    return $self->{chr_page}->nes_true_color($c);
  }

  sub get_cmap {
    my $self = shift;
    return $self->{gd} if defined $self->{gd};

    my @pal;
    if (ref $self->{palette}) {
      @pal = @{$self->{palette}};
    } else {
      push(@pal, $self->{palette});
    }
    $self->{chr_page}->gd_page($self->{page}, @pal);
    $self->{gd} = $self->{chr_page}->get_gd();
    return $self->{gd};
  }

  sub new {
    my $class = shift;
    my $self =  $class->SUPER::new(@_);
    my %opt = %{$self->{orig_opt}};
    if (exists $opt{file}) {
      $self->{chr_page} = NES::ChrPage->new($opt{file});
    } else {
      $self->{chr_page} = JRF::FlagSemaphore->get_cvar()->{CHR_PAGE};
    }
    if (exists $opt{nes}) {
      $self->{page} = $opt{nes};
    }
    if (exists $opt{palette}) {
      if ($opt{palette} =~ /,/) {
	$self->{palette} = [split(/\s*,\s*/, $opt{palette})];
      } else {
	$self->{palette} = $opt{palette};
      }
    }
    return $self;
  }
}


{
  package JRF::Utils;
  use base qw(Exporter);

  use Encode qw();
  use Carp;
#  use JRF::Resource;

  our @EXPORT_OK = qw(encode_con decode_con encode_fn decode_fn
		      encode_u decode_u parse_int unpack_bits pack_bits
		      xd dump);
  our %EXPORT_TAGS = (all => \@EXPORT_OK, debug => [qw(xd dump)]);

  sub encode_con {
    my ($s) = @_;
    return Encode::encode($Resource::CONSOLE_ENCODING, $s,
			  sub { sprintf "\\u{%X}", $_[0] });
  }

  sub decode_con {
    my ($s) = @_;
    return Encode::decode($Resource::CONSOLE_ENCODING, $s,
			  sub { sprintf "\\u{%X}", $_[0] });
  }

  sub encode_fn {
    my ($s) = @_;
    return Encode::encode($Resource::FILENAME_ENCODING, $s,
			  sub { sprintf "\\u{%X}", $_[0] });
  }

  sub decode_fn {
    my ($s) = @_;
    return Encode::decode($Resource::FILENAME_ENCODING, $s,
			  sub { sprintf "\\u{%X}", $_[0] });
  }

  sub encode_u {
    my ($s) = @_;
    return Encode::encode("utf8", $s,
			  sub { sprintf "\\u{%X}", $_[0] });
  }

  sub decode_u {
    my ($s) = @_;
    return Encode::decode("utf8", $s,
			  sub { sprintf "\\u{%X}", $_[0] });
  }

  sub unpack_bits {
    my ($origspec, $origbits) = @_;
    my @r;
    my $c = "B";
    my $len = 0;
    my $spec = reverse($origspec);
    my $bits = $origbits;
    while ($spec ne "") {
      my $n = 1;
      if ($spec =~ s/^[01-9]+//) {
	$n = reverse($&);
      }
      $spec =~ s/^.//;
      if ($& eq "b") {
	$c = "b";
      }
      push(@r, $bits & ((1 << $n) - 1));
      $bits = $bits >> $n;
      $len += $n;
    }
    if ($c ne "b") {
      return reverse @r;
    }
    @r = ();
    $spec = $origspec;
    $bits = $origbits;
    while ($spec ne "") {
      my $n = 1;
      my $c = "B";
      if ($spec =~ s/^[^01-9]//) {
	$c = $&;
      }
      if ($spec =~ s/^[01-9]+//) {
	$n = $&;
      }
      if ($c eq "b") {
	push(@r, $bits & ((1 << $n) - 1));
	$bits = $bits >> $n;
      } else {
	push(@r, ($bits >> ($len - $n)) & ((1 << $n) - 1));
      }
      $len -= $n;
    }
    return @r;
  }

  sub pack_bits {
    my ($spec, @bits) = @_;
    my @r;
    my $len = 0;
    my $bits = 0;
    while (@bits) {
      my $b = shift(@bits);
      my $n = 1;
      my $c = "B";
      if ($spec =~ s/^[^01-9]//) {
	$c = $&;
      }
      if ($spec =~ s/^[01-9]+//) {
	$n = $&;
      }
      $b = $b & ((1 << $n) - 1);
      if ($c eq "b") {
	$bits = $bits | ($b << $len);
      } else {
	$bits = ($bits << $n) | $b;
      }
      $len += $n;
    }
    return $bits;
  }

  sub parse_int {
    my ($s, $radix) = @_; # $radix is a preferred radix, not forced.
    $s =~ s/^\s+//s;
    my $neg = ($s =~ s/^\-\s*//)? -1 : 1;
    if ($s =~ /^0x([01-9A-F_]+)/i || $s =~ /^0x\{([01-9A-F_]+)\}/i || $s =~ /^0x\(([01-9A-F_]+)\)/i) {
      my $x = $1 || "0";
      $x =~ s/_//g;
      return $neg * hex($x);
    } elsif ($s =~ /^0o([01-7_]+)/i) {
      my $x = $1;
      $x =~ s/_//g;
      return $neg * oct($x);
    } elsif (defined $radix && $radix != 16 && $s =~ /^0d([01-9_]+)/i) {
      my $x = $1 || "0";
      $x =~ s/_//g;
      return $neg * int($x);
    } elsif (defined $radix && $radix != 16 && $s =~ /^0b([01_]+)/i) {
      my $x = $1 || "0";
      $x =~ s/_//g;
      return $neg * ord(pack("b" . length($x), scalar reverse($x)));
    } elsif ($s =~ /^0o\{([01-7_]+)\}/i || $s =~ /^0o\(([01-7_]+)\)/i) {
      my $x = $1 || "0";
      $x =~ s/_//g;
      return $neg * oct($x);
    } elsif ($s =~ /^0d\{([01-9_]+)\}/i || $s =~ /^0d\(([01-9_]+)\)/i) {
      my $x = $1 || "0";
      $x =~ s/_//g;
      return $neg * int($x);
    } elsif ($s =~ /^0b\{([01_]+)\}/i || $s =~ /^0b\(([01_]+)\)/i) {
      my $x = $1 || "0";
      $x =~ s/_//g;
      return $neg * ord(pack("b" . length($x), scalar reverse($x)));
    } elsif (! defined $radix || ! $radix || $radix == 10) {
      $s =~ s/_//g;
      $s = $s || "0";
      return $neg * int($s);
    }
    if ($radix == 16) {
      $s =~ s/_//g;
      $s = $s || "0";
      return $neg * hex($s);
    } elsif ($radix == 8) {
      $s =~ s/_//g;
      $s = $s || "0";
      return $neg * oct($s);
    } elsif ($radix == 2) {
      $s =~ s/_//g;
      $s = $s || "0";
      $s =~ /^([01]+)/;
      return $neg * ord(pack("b" . length($1), scalar reverse($1)));
    } else {
      carp (__PACKAGE__ . "::parse_int : supports radix of 2, 8, 10 and 16 only, not $radix.");
      return undef;
    }
  }

  sub dump {
    use Data::Dumper;
    print Dumper(@_);
  }

  sub xd {
    my @r;
    foreach my $s (@_) {
      push(@r, join(" ", map {scalar unpack("H2", $_)} split("", $s)));
    }
    if (wantarray()) {
      return @r;
    } else {
      return join(" ", @r);
    }
  }
}


{
  package JRF::FlagSemaphore::NgbCommand;
  use base qw(Naggy::Backend::CommandInterface);
  provide_class __PACKAGE__;


  __PACKAGE__->extend_template
    (
     jc => undef,

     allow_trl_init => 0,
    );

  __PACKAGE__->extend_cvar
    (
     ABBREV_gif_font => "gif_font",
     ABBREV_sav_default => \&sav_default, # CODE ref for debug.
     ABBREV_trl_init => "trl_init",
     ABBREV_giffont => "gif_font",
     ABBREV_savdefault => "sav_default",
     ABBREV_trlinit => "trl_init",
     ABBREV_find_resource => "find_resource",
    );

  sub new {
    my $class = shift;
    my ($jc, @rest) = @_;
    my $obj =  $class->SUPER::new($jc->{ngb}, @_);
    $obj->{jc} = $jc;
    return $obj;
  }

  sub gif_font {
    my $self = shift;
    my $ngb = $self->{ngb};
    my $JC = $self->{jc};
    if (@_ < 2) {
      $ngb->rerror("illegal arguments.");
      return;
    }
    my $trl = shift;
    if (@_ % 2 == 1) {
      unshift(@_, "file");
    }
    my %opt = @_;
    my $gf;
    if (exists $opt{"--nes"} || exists $opt{nes}) {
      $gf = GifFont::JRFSemaphoreRom->new(@_);
    } else {
      $gf = GifFont->new(@_);
    }
    if (defined $gf) {
      $JC->{gif_font}->{$trl} = $gf;
      $ngb->rprint("# begin unit\nok\n# end\n");
    } else {
      $ngb->rerror("load failure.");
    }
  }

  sub sav_default {
    my $self = shift;
    my $ngb = $self->{ngb};
    my $JC = $self->{jc};
    if (@_ < 2 && @_ % 2 != 1) {
      $ngb->rerror("illegal arguments.");
      return;
    }
    my $trl = shift;
    $JC->{sav_default}->{$trl} = [@_];
    $ngb->rprint("# begin unit\nok\n# end\n");
  }

  sub trl_init {
    my $self = shift;
    my $ngb = $self->{ngb};
    my $JC = $self->{jc};
    if (! $self->{allow_trl_init}) {
      $ngb->rerror("forbidden.");
      return;
    }
    if (@_ != 2) {
      $ngb->rerror("illegal arguments.");
      return;
    }
    my $trl = shift;
    my $f = shift;

    $JC->{trl_init}->{$trl} = $f;
    $ngb->rprint("# begin unit\nok\n# end\n");
  }

  sub find_resource {
    my $self = shift;
    my $ngb = $self->{ngb};
    my ($file, @path) = @_;
    if (@_ != 1) {
      $ngb->rerror("illegal arguments. Usage: find-resource file");
      return;
    }
    my $r = locate_file_resource($file);
    if (defined $r) {
      $ngb->rprint("# begin string\n" . Naggy::escape_string($r) . "\n# end\n");
    } else {
      $ngb->rerror("$file doesn't exists in the path.");
    }
  }
}


{
  package JRF::FlagSemaphore::Command;
  use base qw(JRF::MyOO);
  provide_class __PACKAGE__;

  use Encode qw();
  use IO::Handle;
  use Pod::Usage;
  use Getopt::Long qw();
  use Carp;

  __PACKAGE__->extend_template
    (
     j => undef,
     translit => undef,
     ngb => undef,
     gif_font => {},
     sav_default => {},
     trl_init => {},
     ngb_init_string => undef,

     main_mode => undef,
     trl => undef,
     output_mode => 0,
     mode_opt => [],
     output => undef,
     input => undef,
    );

  our $NGB_INIT = "00_register_trl_init.nginit";
  our %GIF_FONT;
  our %SAV_DEFAULT;
  our %TRL_NGINIT;

  sub new {
    my $class = shift;
    my $obj =  $class->SUPER::new(@_);
    $obj->{j} = JRF::FlagSemaphore->new();
    $obj->{translit} = Naggy::Backend::Translit->new();
    %{$obj->{gif_font}} = %GIF_FONT;
    %{$obj->{sav_default}} = %SAV_DEFAULT;
    %{$obj->{trl_init}} = %TRL_NGINIT;
    return $obj;
  }

  sub prepare_trl {
    my $self = shift;
    my ($trlid, $trlname) = @_;
    my $T = $self->{translit};
    return $trlname if exists $T->{table}->{$trlname};
    if (! defined $self->{ngb}) {
      $self->init_naggy_backend();
      croak "Naggy::Backend: failed to invoke." if ! defined $self->{ngb};
    }
    return $trlname if exists $T->{table}->{$trlname};
    croak "$trlname: not found." if ! exists $self->{trl_init}->{$trlid};
    my $fn = locate_file_resource($self->{trl_init}->{$trlid});
    $self->{ngb}->load_init_file($fn);
    return $trlname if exists $T->{table}->{$trlname};
    croak "$trlname: not found.";
  }

  sub init_naggy_backend {
    my $self = shift;
    my $ngb = $self->{ngb};
    if (! defined $ngb) {
      require_class "Naggy::Backend";
      $ngb = Naggy::Backend->new(translit => $self->{translit});
      return undef if ! defined $ngb;
      $self->{ngb} = $ngb;
    }
    my $ci = JRF::FlagSemaphore::NgbCommand->new($self);
    foreach my $k (%{(ref $ci)->get_cvar()}) {
      if ($k =~ /^ABBREV_/) {
	my $n = $';
	$n =~ s/_/\-/g;
	$ngb->{command}->{$n} = $ci;
      }
    }
    $ngb->{INIT_VAR}->{"HOME"} = $ENV{HOME};

    my $init = $self->{ngb_init_string};
    if (! defined $init && defined $NGB_INIT) {
      my $fn = locate_file_resource($NGB_INIT);
      open(my $fh, "<", $fn) or die "$NGB_INIT: $!";
      $init = join("", <$fh>);
      close($fh);
    }
    if (defined $init) {
      $ci->{allow_trl_init} = 1;
      $ngb->load_init_file_from_string($init);
      $ci->{allow_trl_init} = 0;
    }

    return $ngb;
  }

  sub encode {
    my $self = shift;
    my $J = $self->{j};
    my $T = $self->{translit};
    my $s = shift;
    my %opt
      = (
	 trl => $self->{trl},
	 @_
	);
    my $init = "jrf ";
    $init .= "t $opt{trl} " if defined $opt{trl};
    $init .= "c ";
    if (defined $opt{trl}) {
      my $trl = $opt{trl} . "-alpha";
      $self->prepare_trl($opt{trl}, $trl);
      $s = $T->translit($trl, $s);
    }
    $s = $init . $s;
    return $J->string_to_flag_code($s, @{$self->{mode_opt}}, flush => 1, @_);
  }

  sub decode {
    my $self = shift;
    my $J = $self->{j};
    my $T = $self->{translit};
    my $s = shift;
    my %opt
      = (
	 trl => $self->{trl},
	 @_
	);
    $s = $J->flag_code_to_string($s, @{$self->{mode_opt}}, flush => 1, @_);
    my $init;
    if ($s =~ /^jrf / && $s =~ / c /) {
      $init = $` . $&;
      $s = $';
    }
    if (! defined $init) {
      warn "The input seems not to be flag codes, but proceeding.\n";
    } else {
      $init =~ s/^jrf //;
      $init =~ s/ ?c $//;
      my %o = split(/\s+/, $init);
      if (exists $o{t} && ! defined $opt{trl}) {
	$opt{trl} = $o{t};
	$self->{trl} = $o{t};
      }
    }
    if (defined $opt{trl}) {
      my $trl =  "alpha-" . $opt{trl};
      $self->prepare_trl($opt{trl}, $trl);
      $s = $T->translit($trl, $s);
    }
    return $s;
  }

  sub translit_encode {
    my $self = shift;
    my $T = $self->{translit};
    my $s = shift;
    my %opt
      = (
	 trl => $self->{trl},
	 @_
	);
    if (defined $opt{trl}) {
      my $trl = "alpha-" . $opt{trl};
      $self->prepare_trl($opt{trl}, $trl);
      $s = $T->translit($trl, $s);
    } else {
      die "No transliteration table was specifiled.";
    }
    return $s;
  }

  sub translit_decode {
    my $self = shift;
    my $T = $self->{translit};
    my $s = shift;
    my %opt
      = (
	 trl => $self->{trl},
	 @_
	);
    if (defined $opt{trl}) {
      my $trl = $opt{trl} . "-alpha";
      $self->prepare_trl($opt{trl}, $trl);
      $s = $T->translit($trl, $s);
    } else {
      die "No transliteration table was specifiled.";
    }
    return $s;
  }

  sub text_gif {
    my $self = shift;
    my $s = shift;
    my %opt
      = (
	 trl => $self->{trl},
	 @_
	);
    $opt{trl} = "ascii" if ! defined $opt{trl};
    $self->prepare_trl($opt{trl}, "ascii-" . $opt{trl})
      if ! exists $self->{gif_font}->{$opt{trl}};
    if (exists $self->{gif_font}->{$opt{trl}}) {
      my $gf = $self->{gif_font}->{$opt{trl}};
      return $gf->text_gd($s, @{$self->{mode_opt}}, @_)->gif();
    } else {
      die "No transliteration table was specifiled.";
    }
  }

  sub flag_gif {
    my $self = shift;
    my $J = $self->{j};
    my $s = shift;
    my $sailor = $J->new_sailor(@{$self->{mode_opt}}, @_);
    return $sailor->flag_gif($s . "\x44", @{$self->{mode_opt}}, @_);
  }

  sub generate_sav {
    my $self = shift;
    my $J = $self->{j};
    my $s = shift;
    my @opt= @_;
    my %opt
      = (
	 trl => $self->{trl},
	 @_
	);
    if (defined $opt{trl} && exists $self->{sav_default}->{$opt{trl}}) {
      @opt = (@{$self->{sav_default}->{$opt{trl}}}, @opt);
    }
    return $J->generate_sav($s, @{$self->{mode_opt}}, @opt);
  }

  sub process_arguments {
    my $self = shift;
    my @args = @_;

    my $main_mode;
    my $gif_mode = 0;
    my $trl = undef;
    my $flag_gif_opt = 0;
    my $text_gif_opt = 0;
    my $sav_opt = 0;
    my $ngb_init = undef;;
    my @mode_opt;
    my $output = undef;

    $main_mode = "encode";
    if (@args != 0) {
      my $c = $args[0];
      if ($c eq "-t" || $c eq "--translit") {
	shift(@args);
	$main_mode = "translit_encode";
	if (@args > 0) {
	  $c = $args[0];
	  if ($c eq "-c" || $c eq "--encode") {
	    shift(@args);
	  } elsif ($c eq "-u" || $c eq "--decode") {
	    shift(@args);
	    $main_mode = "translit_decode";
	  }
	}
      } elsif ($c eq "-c" || $c eq "--encode") {
	shift(@args);
      } elsif ($c eq "-u" || $c eq "--decode") {
	shift(@args);
	$main_mode = "decode";
      } elsif ($c eq "-n" || $c eq "--raw") {
	shift(@args);
	$main_mode = "text";
      } elsif ($c eq "--sav") {
	shift(@args);
	$main_mode = "sav";
      } elsif ($c eq "--rom") {
	shift(@args);
	$main_mode = "rom";
      }
    }

    Getopt::Long::Configure("bundling", "auto_version", "no_ignore_case");
    Getopt::Long::GetOptionsFromArray
       (\@args,
	"console-encoding=s" => \$Resource::CONSOLE_ENCODING,
	"filename-encoding=s" => \$Resource::FILENAME_ENCODING,
	"o|output=s" => \$output,
	"T|trl=s"  => \$trl,
	"trl-init=s"  => \$ngb_init,
	"flag-gif"  => \$flag_gif_opt,
	"text-gif"  => \$text_gif_opt,
	"sav"  => \$sav_opt,
	"g"  => \$gif_mode,
	"D|define=s@" => \@mode_opt,
	"man" => sub {pod2usage(-verbose => 2)},
	"h|?" => sub {pod2usage(-verbose => 0, -output=>\*STDOUT, 
				-exitval => 1)},
	"help" => sub {pod2usage(1)},
       ) or pod2usage(-verbose => 0);
    for (my $i = 0; $i < @args; $i++) {
      $args[$i] = decode_con($args[$i]);
    }
    my @r;
    foreach my $kv (@mode_opt) {
      my ($k, $v) = split(/[= ]/, decode_con($kv), 2);
      push(@r, $k, $v);
    }
    @mode_opt = @r;

    $gif_mode = "F" if $flag_gif_opt;
    $gif_mode = "T" if $text_gif_opt;
    if ($flag_gif_opt && $main_mode ne "encode" && $main_mode ne "text") {
      die "--flag-gif must be used with -c or -n .";
    }
    if ($text_gif_opt && $main_mode ne "decode" && $main_mode ne "text"
	&& $main_mode ne "translit_decode" && $main_mode ne "translit_encode") {
      die "--text-gif must be used with -u or -n or -t -u .";
    }
    if ($gif_mode && $main_mode eq "encode") {
      $gif_mode = "F";
    }
    if ($gif_mode && $main_mode =~ /decode/) {
      $gif_mode = "T";
    }
    if ($gif_mode && $gif_mode ne "T" && $gif_mode ne "F") {
      $gif_mode = "T";
    }
    $gif_mode = "S" if $sav_opt;

    die "Too many arguments." if @args > 1;

    $self->{main_mode} = $main_mode;
    $self->{output_mode} = $gif_mode;
    $self->{trl} = decode_con($trl) if defined $trl;
    $self->{mode_opt} = \@mode_opt;
    $self->{output} = decode_con($output) if defined $output;
    $self->{input} = $args[0] if @args;

    if (defined $ngb_init) {
      my $fn = decocde_con($ngb_init);
      open(my $fh, "<", encode_fn($fn))	or die "$fn: $!";
      my $self->{ngb_init_string} = join("", <$fh>);
      close($fh);
    }
  }

  sub command_loop {
    my $self = shift;
    my $J = $self->{j};
    my $T = $self->{translit};
    my $main_mode = $self->{main_mode};
    my $output_mode = $self->{output_mode};

    STDOUT->autoflush(1);
    binmode(STDERR, ":" . $Resource::CONSOLE_ENCODING);

    my $rawinput;
    if (! grep {$main_mode eq $_} qw(rom)) {
      if (defined $self->{input}) {
	my $fn = $self->{input};
	open(my $fh, "<", encode_fn($fn)) or die "$fn: $!";
	binmode($fh, ":raw");
	$rawinput = join("", <$fh>);
	close($fh);
      } else {
	binmode(STDIN, ":raw");
	$rawinput = join("", <STDIN>);
      }
    }

    my $output;
    if ($main_mode eq "encode") {
      $output = $self->encode_raw($rawinput);
    } elsif ($main_mode eq "decode") {
      $output = $self->decode_raw($rawinput);
    } elsif ($main_mode eq "translit_encode") {
      $output = $self->translit_encode_raw($rawinput);
    } elsif ($main_mode eq "translit_decode") {
      $output = $self->translit_decode_raw($rawinput);
    } elsif ($main_mode eq "text") {
      $output = $rawinput;
    } elsif ($main_mode eq "sav") {
      $output = $self->generate_sav_raw($rawinput);
    } else {
      die "$main_mode: Future Work!";
    }

    if ($output_mode) {
      if ($output_mode eq "S") {
	$output = $self->generate_sav_raw($output);
      } elsif ($output_mode eq "F") {
	$output = $self->flag_gif_raw($output);
      } else {
	$output = $self->text_gif_raw($output);
      }
    }
    if (defined $self->{output}) {
      my $fn = $self->{output};
      open(my $fh, ">", encode_fn($fn)) or die "$fn: $!";
      binmode($fh, ":raw");
      print $fh $output;
      close($fh);
    } else {
      binmode(STDOUT, ":raw");
      print $output;
    }
    exit(0);
  }

  sub encode_raw {
    my $S = shift; my $s = shift; 
    return $S->encode(decode_u($s), @_);
  }
  sub decode_raw {
    my $S = shift; my $s = shift; 
    return encode_u($S->decode($s, @_));
  }
  sub translit_encode_raw {
    my $S = shift; my $s = shift; 
    return encode_u($S->translit_encode(decode_u($s), @_));
  }
  sub translit_decode_raw {
    my $S = shift; my $s = shift; 
    return encode_u($S->translit_decode(decode_u($s), @_));
  }
  sub text_gif_raw {
    my $S = shift; my $s = shift; 
    return $S->text_gif(decode_u($s), @_);
  }
  sub flag_gif_raw {
    my $S = shift; my $s = shift; 
    return $S->flag_gif($s, @_);
  }
  sub generate_sav_raw {
    my $S = shift; my $s = shift; 
    return $S->generate_sav(decode_u($s), @_);
  }

 ClassInit:
  {
    $GIF_FONT{"nes8x8"} = GifFont::JRFSemaphoreRom->new
      (nes => 0,
       a => "FF61-FF9F:A1", a => "FFE5:A0",  # japanese hwkata.
       a => "5D0-5EA:FD-E3", a => "5C3:E2",  # hebrew.
       a => "5BC:E1", a => "20AA:E0",
      );
    $GIF_FONT{"nes8x8_rtol"} = GifFont::JRFSemaphoreRom->new
      (nes => 0, direction => "right-to-left",
       a => "FF61-FF9F:A1", a => "FFE5:A0",
       a => "5D0-5EA:FD-E3", a => "5C3:E2",
       a => "5BC:E1", a => "20AA:E0",
      );
    $GIF_FONT{"nes8x16a"} = GifFont::JRFSemaphoreRom->new
      (nes => 1, size => "8x16",
       a => "FF61-FF9F:A1", a => "FFE5:A0",
       a => "5D0-5EA:FD-E3", a => "5C3:E2",
       a => "5BC:E1", a => "20AA:E0",
      );
    $GIF_FONT{"nes8x16b"} = GifFont::JRFSemaphoreRom->new
      (nes => 3, size => "8x16",
       a => "FF61-FF9F:A1", a => "FFE5:A0",
       a => "5D0-5EA:FD-E3", a => "5C3:E2",
       a => "5BC:E1", a => "20AA:E0",
      );
    $GIF_FONT{"nes8x16b_rtol"} = GifFont::JRFSemaphoreRom->new
      (nes => 3, size => "8x16", d => "r-to-l",
       a => "FF61-FF9F:A1", a => "FFE5:A0",
       a => "5D0-5EA:FD-E3", a => "5C3:E2",
       a => "5BC:E1", a => "20AA:E0",
      );
    $GIF_FONT{"j0heb.nes"} = $GIF_FONT{"nes8x8_rtol"};
    $GIF_FONT{"j0ja.hw"} = $GIF_FONT{"nes8x8"};
    $GIF_FONT{"ascii"} = $GIF_FONT{"nes8x8"};
    $GIF_FONT{"alpha"} = $GIF_FONT{"nes8x8"};
    $SAV_DEFAULT{"j0heb.nes"}
      = [righttoleft => 1, toprintable => 0, font => 5];
    $SAV_DEFAULT{"j0ja.hw"} = [toprintable => 0];
  }
}

MAIN:
if (! defined $main::IN_TEST || ! $main::IN_TEST)
{
  package Main;

  if (@ARGV > 0 &&
      ($ARGV[0] eq "--naggy-backend" || $ARGV[0] eq "--ngb")) {
    shift(@ARGV);
    require_class "Naggy::Backend";
    my $JC = JRF::FlagSemaphore::Command->new();
    my $N = Naggy::Backend::Command->new(@ARGV);
    my $ngb = $N->{backend};
    $JC->{ngb} = $ngb;
    $JC->init_naggy_backend();
    $JC->{ngb}->{command}->{"trl-init"}->{allow_trl_init} = 1;
    $N->command_loop();
  } else {
    my $JC = JRF::FlagSemaphore::Command->new();
    $JC->process_arguments(@ARGV);
    $JC->command_loop();
  }
}

BEGIN {
    $ENV{"PERLDOC"} = "" if ! exists $ENV{"PERLDOC"};
    $ENV{"PERLDOC"} .= " " if $ENV{"PERLDOC"} ne "";
    $ENV{"PERLDOC"} .= "-wcenter:'JRF Flag Semaphore Command Manual'";
}

=pod

=head1	NAME

jrf_samephore.pl - the encoding/decoding program of a flag semaphore.

=head1	SYNOPSIS

B<jrf_semaphore.pl> -c [--T TRL_NAME] [--flag-gif|-g] INPUT_FILE

B<jrf_semaphore.pl> -u [--T TRL_NAME] [--text-gif|-g|--sav] INPUT_FILE

B<jrf_semaphore.pl> -t -c [--T TRL_NAME] [--sav] INPUT_FILE

B<jrf_semaphore.pl> -t -u [--T TRL_NAME] [--text-gif|-g|--sav] INPUT_FILE

B<jrf_semaphore.pl> -n [--flag-gif|--text-gif] INPUT_FILE

B<jrf_semaphore.pl> --sav [--T TRL_NAME] INPUT_FILE

=head1	Options

=over 8

=item B<--help>

shows help message about options.

=item B<--man>

shows man page.

=item B<--version>

shows version infomation.

=item B<-c>

encoding mode. default.

=item B<-u>

decoding mode.

=item B<-t> or B<--translit>

transliteration only.

=item B<-n> or B<--raw>

no encoding. no decoding. no transliteration. but output.

=item B<--sav>

output .SAV file for jrf_semaphore.nes .

=item B<-o> F<OUTPUT> or B<--output> F<OUTPUT>

specifies a filename of output.

=item B<-T> F<TABLE_NAME> or B<--trl> F<TABLE_NAME>

specifies a table name of transliteration.

This is a test version.  You can specify only j0heb.nes or j0ja.hw .

=item B<--flag-gif> or B<-g>

output gif animation of the flag code.

=item B<--text-gif> or B<-g>

output text as a gif.

=item B<--console-encoding> F<encoding>

specifies the encoding for console arguments.

=item B<--filename-encoding> F<encoding>

specifies the encoding for filenames.

=item B<-D> F<opt>=F<value>

specifies a mode option.

=back

=head1	DESCRIPTION

B<This program> is the encoding or decoding program of a flag
semaphore which is originally implemented for NES emulators as LLVM.

=head1 EXAMPLE

$ echo -n br\'syt | perl jrf_semaphore.pl -t -c -T j0heb.nes \
    | perl jrf_semaphore.pl -c -T j0heb.nes \
    | perl jrf_semaphore.pl -u --text-gif > test.gif

=head1	AUTHOR'S LINK

JRF E<lt>http://jrf.cocolog-nifty.com/softwareE<gt>

=head1	LICENSE

I in a provincial state made this program intended to be public-domain. 
But it might be better for you like me to treat this program as such 
under the BSD-License or under the Artistic License.

Within three months after the release of this program, I
especially admit responsibility of effort for rational request of
correction to this program.

=head1	SEE ALSO

L<Encode>

=cut

